/*
 * Licensed to UbiCollab.org under one or more contributor
 * license agreements.  See the NOTICE file distributed
 * with this work for additional information regarding
 * copyright ownership. UbiCollab.org licenses this file
 * to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package no.ntnu.android.toolkit.activities;

import android.content.Intent;
import android.os.Bundle;
import android.support.v4.view.MenuItemCompat;
import android.support.v7.app.ActionBarActivity;
import android.support.v7.widget.SearchView;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ListView;
import no.ntnu.android.toolkit.appstore.AppServerConnection;
import no.ntnu.android.toolkit.appstore.AppServerConnectionStub;
import no.ntnu.android.toolkit.appstore.Serialization;
import no.ntnu.android.toolkit.graphic.adapter.AppListAdapter;
import no.ntnu.android.toolkit.model.HardwareSetup;
import no.ntnu.android.toolkit.model.IdGenerator;
import no.ntnu.android.toolkit.model.IdInterface;
import no.ntnu.android.toolkit.reader.BoardTypeReader;

import java.util.ArrayList;
import java.util.List;

public class AppStore extends ActionBarActivity {

    private AppListAdapter arrayAdapter;
    private AppServerConnection appServerConnection;
    private List<HardwareSetup> hardwareSetups;


    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.appstore);
        getActionBar().setDisplayHomeAsUpEnabled(true);
        BoardTypeReader boardTypeReader = new BoardTypeReader(this);
        boardTypeReader.getDevices();

        appServerConnection = new AppServerConnectionStub();
        initializeDifferentBoardTypesFromServer();


        final ListView list = (ListView)findViewById(R.id.appStoreList);
        arrayAdapter = new AppListAdapter(this, android.R.layout.simple_list_item_1, hardwareSetups);
        list.setAdapter(arrayAdapter);
        list.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> adapterView, View view, int i, long l) {
                redirectAppStoreConfig(((HardwareSetup)list.getItemAtPosition(i)).getName());
            }
        });
        list.setTextFilterEnabled(true);
    }

    private void initializeDifferentBoardTypesFromServer() {
        searchForCorrespondingBoard((List<String>)Serialization.deserializeObject(appServerConnection.getAllBoardTypeNames()));
    }

    private void searchForCorrespondingBoard(List<String> boardNames) {
        hardwareSetups = new ArrayList<HardwareSetup>();
        for (IdInterface idInterface : IdGenerator.getInstance().getAllObjects()) {
            if (idInterface instanceof HardwareSetup) {
                for (String hardwareSetupName : boardNames) {
                    if(((HardwareSetup) idInterface).getName() != null){
                        if (((HardwareSetup) idInterface).getName().equalsIgnoreCase(hardwareSetupName) && hardwareSetupContains((HardwareSetup) idInterface)) {
                            hardwareSetups.add((HardwareSetup) idInterface);
                        }
                    }
                }
            }
        }
    }
    private boolean hardwareSetupContains(HardwareSetup hardwareSetup) {
        for (HardwareSetup hardwareSetup1 : hardwareSetups) {
            if (hardwareSetup1.getName().equalsIgnoreCase(hardwareSetup.getName())) {
                return false;
            }
        }
        return true;
    }

    private void redirectAppStoreConfig(String boardname) {
        ArrayList<String> query = new ArrayList<String>();
        query.add(boardname);
        Intent appstoreConfigIntent = new Intent(this, AppStoreConfiguration.class);
        appstoreConfigIntent.putStringArrayListExtra("query", query);
        startActivity(appstoreConfigIntent);
    }

    private void filter(String query) {
        arrayAdapter.getFilter().filter(query);
        arrayAdapter.notifyDataSetChanged();
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.search_menu, menu);
        setTitle(getString(R.string.appStoreBoards));
        MenuItem searchItem = menu.findItem(R.id.appStore_search);
        SearchView searchView = (SearchView) MenuItemCompat.getActionView(searchItem);
        searchView.setOnQueryTextListener(new SearchView.OnQueryTextListener() {
            @Override
            public boolean onQueryTextChange(String s) {
                filter(s);
                return false;
            }

            @Override
            public boolean onQueryTextSubmit(String s) {
                return false;
            }
        });

        MenuItemCompat.setOnActionExpandListener(searchItem, new MenuItemCompat.OnActionExpandListener() {
            @Override
            public boolean onMenuItemActionCollapse(MenuItem item) {
                // Do something when collapsed
                return true;  // Return true to collapse action view
            }

            @Override
            public boolean onMenuItemActionExpand(MenuItem item) {
                // Do something when expanded
                return true;  // Return true to expand action view
            }
        });

        return super.onCreateOptionsMenu(menu);
    }
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        if(item.getItemId() == android.R.id.home) {
            Intent intent = new Intent(this, MyActivity.class);
            intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
            this.startActivity(intent);
        }
        return super.onOptionsItemSelected(item);
    }
}