/*
 * Licensed to UbiCollab.org under one or more contributor
 * license agreements.  See the NOTICE file distributed
 * with this work for additional information regarding
 * copyright ownership. UbiCollab.org licenses this file
 * to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package no.ntnu.android.toolkit.activities;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ListView;
import android.widget.Toast;
import no.ntnu.android.toolkit.appstore.*;
import no.ntnu.android.toolkit.graphic.adapter.AppsAdapter;
import no.ntnu.android.toolkit.model.Model;

import java.util.List;

public class AppStoreConfiguration extends Activity {

    private AppServerConnection appServerConnection;
    private AppsAdapter adapter;
    private List<SavedToolkitApplication> applications;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.appstoreconfig);
        getActionBar().setDisplayHomeAsUpEnabled(true);
        appServerConnection = new AppServerConnectionStub();
        List<String> query = this.getIntent().getStringArrayListExtra("query");
        boolean exact = this.getIntent().getBooleanExtra("exact", false);
        applications = handleQuery(query, exact);

        final ListView list = (ListView)findViewById(R.id.appStoreConfigList);
        adapter = new AppsAdapter(this, android.R.layout.simple_list_item_1, applications);
        list.setAdapter(adapter);
        list.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> adapterView, View view, int i, long l) {
                if(list.getItemAtPosition(i) instanceof SavedToolkitApplication) {
                    SavedToolkitApplication application = (SavedToolkitApplication)list.getItemAtPosition(i);
                    Model.updateModel(application, AppStoreConfiguration.this);
                    redirectConfigRules(application.getConnectorConfigurationPoints());
                }
            }
        });
    }

    private List handleQuery(List<String> query, boolean exact) {
        List result = null;
        if(query != null) {
            setTitle(getString(R.string.appStoreApp));
            byte[] serialNames = Serialization.serializeObject(query);
            byte[] byteResult;
            if (exact) {
                byteResult = appServerConnection.getExactHardwareNames((serialNames));
            } else {
                byteResult = appServerConnection.getApplicationsIncludingName((serialNames));
            }
            result = (List<SavedToolkitApplication>)Serialization.deserializeObject(byteResult);
        }
        if (result == null || result.size() == 0) {
            Toast.makeText(getApplicationContext(), "No results found", Toast.LENGTH_SHORT).show();
        }
        return result;
    }

    private void redirectConfigRules(List<List<ConfigurationPoint>> configurationPointList) {
        Intent configureRulesIntent = new Intent(this, ConfigureRules.class);
        configureRulesIntent.putExtra("command", "modelUpdated");
        configureRulesIntent.putExtra("configurationPointList", Serialization.serializeObject(configurationPointList));
        startActivity(configureRulesIntent);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        if(item.getItemId() == android.R.id.home) {
            Intent intent = new Intent(this, AppStore.class);
            intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
            this.startActivity(intent);
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    public void onBackPressed() {
        Intent intent = new Intent(this, MyActivity.class);
        intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
        this.startActivity(intent);
    }
}