/*
 * Licensed to UbiCollab.org under one or more contributor
 * license agreements.  See the NOTICE file distributed
 * with this work for additional information regarding
 * copyright ownership. UbiCollab.org licenses this file
 * to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package no.ntnu.android.toolkit.activities;

import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.widget.LinearLayout;
import no.ntnu.android.toolkit.bluetooth.DiscoverDevices;
import no.ntnu.android.toolkit.model.Model;

public class Bluetooth extends Activity {
    private LinearLayout linearLayout;
    private DiscoverDevices discoverDevices;
    private Bluetooth bluetoothActivity;
    private int TIME_TO_KEEP_SEARCHING_FOR_BLUETOOTH = 5000;
    private MenuItem refreshIcon;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.bluetooth);
        bluetoothActivity = this;
        getActionBar().setTitle("Choose board for " + Model.getInstance().getActiveBoard().getName());
        linearLayout = (LinearLayout)findViewById(R.id.bluetoothLinearLayout);
        discoverDevices = new DiscoverDevices(this, linearLayout, android.R.layout.simple_list_item_1);
        discoverDevices.initiateSearch();
    }

    @Override
    public void onBackPressed() {
        try{
            discoverDevices.unregisterBroadcastReceiver();
        } catch (Exception e){
            e.printStackTrace();
        }
        BluetoothAdapter bluetoothAdapter;
        if((bluetoothAdapter = discoverDevices.getBluetoothAdapter()) != null){
            bluetoothAdapter.cancelDiscovery();
        }
        finish();
        super.onBackPressed();
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater menuInflater = getMenuInflater();
        menuInflater.inflate(R.menu.bluetooth_menu, menu);
        return super.onCreateOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()){
            case R.id.bluetooth_menuRefresh:
                refreshIcon = item;
                discoverDevices.initiateSearch();
                item.setActionView(R.layout.bluetooth_loading_porgress);
                item.expandActionView();
                startNewTimer();
                break;
        }
        return super.onOptionsItemSelected(item);
    }

    private void startNewTimer(){
        new Thread(new Runnable() {
            @Override
            public void run() {
                long timeNow = System.currentTimeMillis();
                long currentTime = System.currentTimeMillis();
                while(currentTime + TIME_TO_KEEP_SEARCHING_FOR_BLUETOOTH > timeNow){
                    timeNow = System.currentTimeMillis();
                }
                bluetoothActivity.runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        refreshIcon.collapseActionView();
                        refreshIcon.setActionView(null);
                    }
                });
            }
        }).start();
    }
}
