/*
 * Licensed to UbiCollab.org under one or more contributor
 * license agreements.  See the NOTICE file distributed
 * with this work for additional information regarding
 * copyright ownership. UbiCollab.org licenses this file
 * to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package no.ntnu.android.toolkit.activities;

import android.app.Activity;
import android.content.pm.ActivityInfo;
import android.graphics.Typeface;
import android.os.Bundle;
import android.view.Gravity;
import android.view.MenuItem;
import android.widget.*;
import no.ntnu.android.toolkit.bluetooth.DiscoverDevices;
import no.ntnu.android.toolkit.graphic.adapter.BoardBluetoothAdapter;

public class BoardBluetoothMapping extends Activity {

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.board_bluetooth_mapping);
        setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);
        initiateList();
        getActionBar().setTitle("Pair board and Bluetooth device");
        getActionBar().setDisplayHomeAsUpEnabled(true);
    }

    private void initiateList(){
        ListView boardList = (ListView)findViewById(R.id.board_mapping_listView);
        BoardBluetoothAdapter boardBluetoothAdapter = new BoardBluetoothAdapter(this, android.R.layout.simple_list_item_single_choice, getIntent());
        boardList.setAdapter(boardBluetoothAdapter);

        DiscoverDevices discoverDevices = new DiscoverDevices(this, new BoardBluetoothAdapter(this, android.R.layout.simple_list_item_single_choice, getIntent()), (LinearLayout)findViewById(R.id.boardMappingLayout), getIntent());
        discoverDevices.initiateSearch();
    }
    public void done(){
        finish();
    }

    @Override
    public void onBackPressed() {
        setResult(RESULT_CANCELED);
        done();
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        if(item.getItemId() == android.R.id.home) {
            onBackPressed();
        }
        return super.onOptionsItemSelected(item);
    }

    public void addMenuHeader(LinearLayout linearLayout){
        TableLayout tableLayout = new TableLayout(this);
        TableRow tableRow = new TableRow(this);
        tableRow.setPadding(0, 7, 0,0);
        tableLayout.addView(tableRow);
        TextView textView = new TextView(this);
        textView.setText("Bluetooth device");
        textView.setTextSize(18);
        textView.setGravity(Gravity.LEFT);
        textView.setTypeface(null, Typeface.BOLD);
        textView.setLayoutParams(new TableRow.LayoutParams(TableLayout.LayoutParams.WRAP_CONTENT, TableLayout.LayoutParams.WRAP_CONTENT, 1.0f));
        tableRow.addView(textView);

        TextView textView2 = new TextView(this);
        textView2.setGravity(Gravity.LEFT);
        textView2.setTypeface(null, Typeface.BOLD);
        textView2.setTextSize(18);
        textView2.setLayoutParams(new TableRow.LayoutParams(TableLayout.LayoutParams.WRAP_CONTENT, TableLayout.LayoutParams.WRAP_CONTENT, 1.0f));
        textView2.setText("Board");
        tableRow.addView(textView2);
        linearLayout.addView(tableLayout);
    }
}
