/*
 * Licensed to UbiCollab.org under one or more contributor
 * license agreements.  See the NOTICE file distributed
 * with this work for additional information regarding
 * copyright ownership. UbiCollab.org licenses this file
 * to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package no.ntnu.android.toolkit.activities;

import android.app.Activity;
import android.content.Intent;
import android.content.pm.ActivityInfo;
import android.os.Bundle;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ListView;
import no.ntnu.android.toolkit.graphic.adapter.IPSettingsAdapter;
import no.ntnu.android.toolkit.model.Board;
import no.ntnu.android.toolkit.model.Model;
import no.ntnu.android.toolkit.model.WifiSettings;

import java.util.List;

public class ConfigureNetworkSettings extends Activity{

    public void onCreate(Bundle savedInstanceState){
        super.onCreate(savedInstanceState);
        setContentView(R.layout.wifi_and_ip_settings);
        if (getIntent().getStringExtra("extra") != null && getIntent().getStringExtra("extra").equalsIgnoreCase("fromMenu")) {
            initiateBoardList();
        }
        setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);
        getActionBar().setTitle("Network settings");
        getActionBar().setDisplayHomeAsUpEnabled(true);

        Button saveIpAddressesButton = (Button) findViewById(R.id.btnSaveIpAddresses);
        saveIpAddressesButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                if(saveSettings()){
                    Intent returnToUpload = new Intent(getApplicationContext(), ConfigureRules.class);
                    setResult(RESULT_OK, returnToUpload);
                    finish();
                }
            }
        });

        if (Model.getInstance().getWifiSettings() != null) {
            ((EditText)findViewById(R.id.wifiSsid)).setText(Model.getInstance().getWifiSettings().getSsid());
            ((EditText)findViewById(R.id.wifiPassword)).setText(Model.getInstance().getWifiSettings().getPassword());
        }
    }

    private void initiateBoardList(){
        ListView boardList = (ListView)findViewById(R.id.boardIPList);
        IPSettingsAdapter ipSettingsAdapter = new IPSettingsAdapter(ConfigureNetworkSettings.this, android.R.layout.simple_list_item_1);
        boardList.setAdapter(ipSettingsAdapter);
        ipSettingsAdapter.addAll(Model.getInstance().getAllBoardsWithWifi());
        ipSettingsAdapter.notifyDataSetChanged();
    }

    private boolean saveSettings() {
        List<Board> boards = Model.getInstance().getBoards();
        String ssid = ((EditText)findViewById(R.id.wifiSsid)).getEditableText().toString();
        String pass = ((EditText)findViewById(R.id.wifiPassword)).getEditableText().toString();
        Model.getInstance().setWifiSettings(new WifiSettings(ssid, pass));
        for (final Board board : boards) {
            if (board.getWifiDevice() != null) {
                board.getWifiSettings().setSsid(ssid);
                board.getWifiSettings().setPassword(pass);
            }
        }
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        if(item.getItemId() == android.R.id.home) {
            onBackPressed();
        }
        return super.onOptionsItemSelected(item);
    }
}
