/*
 * Licensed to UbiCollab.org under one or more contributor
 * license agreements.  See the NOTICE file distributed
 * with this work for additional information regarding
 * copyright ownership. UbiCollab.org licenses this file
 * to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package no.ntnu.android.toolkit.activities;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.Color;
import android.graphics.Point;
import android.graphics.PorterDuff;
import android.graphics.Typeface;
import android.graphics.drawable.Drawable;
import android.os.Bundle;
import android.support.v4.app.ActionBarDrawerToggle;
import android.support.v4.widget.DrawerLayout;
import android.util.TypedValue;
import android.view.*;
import android.widget.*;
import no.ntnu.android.toolkit.appstore.*;
import no.ntnu.android.toolkit.bluetooth.DiscoverDevices;
import no.ntnu.android.toolkit.graphic.*;
import no.ntnu.android.toolkit.graphic.listener.ConfigRulesOnDragListener;
import no.ntnu.android.toolkit.graphic.listener.ConnectedLinearLayoutOnLongClickListener;
import no.ntnu.android.toolkit.graphic.listener.ConnectedLinearLayoutOnTouchListener;
import no.ntnu.android.toolkit.graphic.listener.DeviceLinearLayoutOnTouchListener;
import no.ntnu.android.toolkit.graphic.view.*;
import no.ntnu.android.toolkit.model.*;
import no.ntnu.android.toolkit.tool.FormatCheck;
import no.ntnu.android.toolkit.uploader.RulesInstaller;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

public class ConfigureRules extends Activity  {

    private AlertDialog alertDialog;
    private ConfigureRules activity;
    private Mode mode;
    private Menu menu;
    private ActionBarDrawerToggle actionBarDrawerToggle;
    private NavigationDrawer navigationDrawer;
    private boolean easyUpload;
    public static final int configurationImageViewSize = 60;
    public static final int configurationImageViewPaddingSize = 5;
    public static final int arrowViewSize = 20;
    public static final int connectorImageViewSize = 25;
    public static final int textSize = 10;
    private DiscoverDevices discoverDevices;
    private static final String connectionNotSupportedMessage = "These boards does not support this connection: All boards are not wifi compatible";

    public enum Mode {
        DRAG, CONNECT
    }

    public void setMode(Mode mode) {
        this.mode = mode;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.configurerules);
        addSettingsIconInMenuBar();
        findViewById(R.id.input_container).setOnDragListener(new ConfigRulesOnDragListener(this));
        findViewById(R.id.output_container).setOnDragListener(new ConfigRulesOnDragListener(this));
        activity = this;
        findViewById(R.id.rule_container).setOnDragListener(new ConfigRulesOnDragListener(this));
        findViewById(R.id.clearX).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                clearRuleContainer();
            }
        });
        setMode(Mode.CONNECT);

        navigationDrawer = new NavigationDrawer(this);
        navigationDrawer.initDrawerLayout();
        actionBarDrawerToggle = navigationDrawer.getActionBarDrawerToggle();

        String commandFromMyActivity = this.getIntent().getStringExtra("command");
        if (commandFromMyActivity != null)  {
            if (commandFromMyActivity.equalsIgnoreCase("startBluetoothDiscovery")
                        && (savedInstanceState == null || Model.getInstance().getBoards().size() == 0)){
                startBluetoothSearch();

            } else if (commandFromMyActivity.equalsIgnoreCase("modelUpdated") && savedInstanceState == null) {
                List<List<ConfigurationPoint>> configurationPointList = (List<List<ConfigurationPoint>>)Serialization.deserializeObject(
                        this.getIntent().getByteArrayExtra("configurationPointList"));
                redrawRuleContainer(configurationPointList);
                DrawerLayout drawerLayout = (DrawerLayout) findViewById(R.id.drawer_layout);
                ListView drawerList = (ListView) findViewById(R.id.left_drawer);
                drawerLayout.closeDrawer(drawerList);
            }
        }
        initiateBoardItems();
    }

    public Mode getMode() {
        return mode;
    }

    private void setAlertDialog(AlertDialog alertDialog) {
        dismissDialog();
        this.alertDialog = alertDialog;
    }

    public AlertDialog getAlertDialog() {
        return alertDialog;
    }

    public void dismissDialog() {
        if (this.alertDialog != null) {
            this.alertDialog.dismiss();
        }
    }

    public void initiateBoardItems() {
        updateMenuBoardNameDisplay();
        ((ViewGroup) findViewById(R.id.input_container)).removeAllViews();
        ((ViewGroup) findViewById(R.id.output_container)).removeAllViews();
        if (Model.getInstance().getActiveBoard() != null && Model.getInstance().getActiveBoard().getHardwareSetup() != null ) {
            Iterator it = Model.getInstance().getActiveBoard().getHardwareSetup().iterator();
            while (it.hasNext()) {
                Device device = (Device)it.next();
                int drawableIcon = ElementConnector.getPictureId(this, device);
                List<Integer> linearLayoutIDs = new ArrayList<Integer>();
                for (DeviceFunction deviceFunction : device.getDeviceFunctions()) {
                    if (deviceFunction.getType() == DeviceFunction.Type.SENSOR && !linearLayoutIDs.contains(R.id.input_container)) {
                        linearLayoutIDs.add(R.id.input_container);
                    } else if (deviceFunction.getType() == DeviceFunction.Type.ACTUATOR && !linearLayoutIDs.contains(R.id.output_container)) {
                        linearLayoutIDs.add(R.id.output_container);
                    }
                }
                for (Integer linearLayoutID : linearLayoutIDs) {
                    String text = device.getText();
                    if (text.contains(":")) {
                        if (text.split(":").length == 1) {
                            text = "";
                        } else {
                            text = text.split(":")[1];
                        }
                    }
                    drawDrawable(drawableIcon, linearLayoutID, device.getId(), text);
                }
            }
        }
    }

    public void startBluetoothSearch() {
        Model.getInstance().clearAllBoards();
        if(discoverDevices == null) {
            discoverDevices = new DiscoverDevices(activity, new LinearLayout(activity),
                android.R.layout.simple_list_item_multiple_choice);
        }
        discoverDevices.showSearchDialog();
        updateNavigationDrawerAdapter();
        easyUpload = true;
    }

    public void updateMenuBoardNameDisplay() {
        if(menu != null && menu.findItem(R.id.boardName) != null &&
                Model.getInstance().getActiveBoard() != null) {
            menu.findItem((R.id.boardName)).setTitle("Board: "
                    + Model.getInstance().getActiveBoard().getName());
        }
    }

    public void drawCLLInRuleContainer(ConfigurationLinearLayout cll) {
        ((ViewGroup)findViewById(R.id.rule_container)).addView(cll);
    }

    public void drawDrawable(Drawable drawable, int linearLayoutID, int objectId, String text) {
        LinearLayout container = new LinearLayout(this);
        container.setId(objectId);
        container.setOrientation(LinearLayout.VERTICAL);
        container.setGravity(Gravity.LEFT);
        ImageView imageView = new ImageView(this);
        imageView.setImageDrawable(drawable);
        final float scale = getResources().getDisplayMetrics().density;
        int pixels = (int) (configurationImageViewSize * scale + 0.5f);
        imageView.setLayoutParams(new LinearLayout.LayoutParams(pixels,
                pixels));
        int paddingPixels = (int) (configurationImageViewPaddingSize * scale + 0.5f);
        imageView.setPadding(paddingPixels, paddingPixels, paddingPixels, paddingPixels);
        int input_container = findViewById(R.id.input_container).getId();
        int output_container = findViewById(R.id.output_container).getId();
        if(findViewById(linearLayoutID).getId() == input_container){
            imageView.setBackgroundResource(R.drawable.rectangleboarder);
        } else if(findViewById(linearLayoutID).getId() == output_container){
            imageView.setBackgroundResource(R.drawable.circleboarder);
        }
        //Adding imageView first
        container.addView(imageView);
        TextView textView = new TextView(this);
        textView.setText("");
        textView.setLayoutParams(new LinearLayout.LayoutParams(pixels, ViewGroup.LayoutParams.WRAP_CONTENT));
        textView.setGravity(Gravity.CENTER);

        //Adding the textView second
        container.addView(textView);
        if(text != null) {
            textView.setText(text);
            textView.setTextSize(TypedValue.COMPLEX_UNIT_SP, textSize);
            //Setting the text black
            textView.setTextColor(Color.parseColor("#000000"));
        }
        container.setOnTouchListener(new DeviceLinearLayoutOnTouchListener());
        ((ViewGroup) findViewById(linearLayoutID)).addView(container);
    }

    public void drawDrawable(int drawableIcon, int linearLayoutID, int objectid, String text) {
        Drawable drawable = getResources().getDrawable(drawableIcon);
        drawDrawable(drawable, linearLayoutID, objectid, text);
    }


    private void installConfigurations() {
        if(Model.getInstance().someRulesRequiresWifi() && !wifiInformationEntered()){
            Intent enterWifiInformation = new Intent(activity, ConfigureNetworkSettings.class);
            activity.startActivityForResult(enterWifiInformation, Constant.INTERCONNECTED_UPLOAD_ADVANCED_REQUEST);
            return;
        }
        if (easyUpload) {
                initiateRulesInstallation(RulesInstaller.InstallType.Interconnected);
        } else {
            showAdvancedInstallationDialog();
        }
    }

    private void showAdvancedInstallationDialog() {
        Button single = new Button(this);
        single.setText(Model.getInstance().getActiveBoard().getName() + "'s configuration only");
        single.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                dismissDialog();
                initiateRulesInstallation(RulesInstaller.InstallType.Single);
            }
        });
        Button connected = new Button(this);
        connected.setText("Normal installation");
        connected.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                dismissDialog();
                initiateRulesInstallation(RulesInstaller.InstallType.Interconnected);
            }
        });
        Button multi = new Button(this);
        multi.setText(Model.getInstance().getActiveBoard().getName() + "'s configuration to " +
                Model.getInstance().getActiveBoard().getHardwareSetup().getName() + " type boards");
        multi.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                dismissDialog();
                initiateRulesInstallation(RulesInstaller.InstallType.MultipleEqual);
            }
        });

        AlertDialog.Builder builder = new AlertDialog.Builder(ConfigureRules.this);
        builder.setTitle("Install configuration");
        LinearLayout linearLayout = new LinearLayout(this);
        linearLayout.setOrientation(LinearLayout.VERTICAL);

        if (Model.getInstance().getBoardsWithRules().size() <= 1) {
            single.setText("Normal installation");
        } else{
            linearLayout.addView(connected);
        }

        linearLayout.addView(single);
        linearLayout.addView(multi);
        builder.setView(linearLayout);
        setAlertDialog(builder.create());

        if(!ConfigureRules.this.isFinishing())
        {
            alertDialog.show();
        }
    }

    private boolean wifiInformationEntered() {
        if (Model.getInstance().getAllBoardsWithWifi().size() > 0) {
            for (Board board : Model.getInstance().getBoardsWithRules()) {
                if (board.getWifiSettings().getSsid() == null || board.getWifiSettings().getSsid().equalsIgnoreCase("")
                        || board.getWifiSettings().getPassword() == null || board.getWifiSettings().getPassword().equalsIgnoreCase("")) {
                    return false;
                }
            }
        }
        return true;
    }

    public void clearRuleContainer(){
        FrameLayout ruleLayout = (FrameLayout)findViewById(R.id.rule_container);
        ruleLayout.removeAllViews();
        Model.getInstance().clearAllRules();
    }

    private void setInitialMenuSettings(Menu menu) {
        getActionBar().setDisplayShowTitleEnabled(false);
        updateMenuBoardNameDisplay();
    }

    private void changeBoardName(final MenuItem item) {
        final EditText view = ElementCreator.createEditTextField(this, Model.getInstance().getActiveBoard().getName(), null);
        AlertDialog.Builder builder = new AlertDialog.Builder(ConfigureRules.this);
        builder.setView(view);
        builder.setTitle("Change name");
        builder.setPositiveButton(R.string.save, new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                String typed = view.getText().toString();
                if(typed.length() > 15) {
                    typed = typed.substring(0, 15);
                }
                Model.getInstance().getActiveBoard().setName(typed);
                Model.getInstance().getActiveBoard().setNameChangedByUser(true);
                updateMenuBoardNameDisplay();
                updateConfigurationLinearLayouts();
                updateNavigationDrawerAdapter();
                dialog.dismiss();
            }
        });
        setAlertDialog(builder.create());
        alertDialog.show();
    }

    private void updateConfigurationLinearLayouts() {
        List<ConfigurationLinearLayout> clls  = getConfigurationLinearLayouts();
        for (ConfigurationLinearLayout cll : clls) {
            String configurationText = "";
            if (cll.getConfiguration().getDevice().getText() != null &&
                    !cll.getConfiguration().getDevice().getText().equals("")) {
                configurationText = cll.getConfiguration().getDevice().getText() + "\n";
            }
            configurationText += cll.getConfiguration().getBoard().getName();
            cll.setText(configurationText);
        }
    }

    public List<ConfigurationLinearLayout> getConfigurationLinearLayouts() {
        List<ConfigurationLinearLayout> clls= new ArrayList<ConfigurationLinearLayout>();
        for (int i = 0; i < ((ViewGroup)findViewById(R.id.rule_container)).getChildCount(); i++) {
            if (((ViewGroup)findViewById(R.id.rule_container)).getChildAt(i) instanceof ConfigurationLinearLayout) {
                clls.add((ConfigurationLinearLayout) ((ViewGroup)findViewById(R.id.rule_container)).getChildAt(i));
            }
        }
        return clls;
    }


    public void configureParameters(final ConfigurationLinearLayout cll){
        AlertDialog.Builder builder = new AlertDialog.Builder(ConfigureRules.this);
        builder.setTitle(R.string.configureDevice);
        LinearLayout layout;
        if (cll.getParameterLayout() == null) {
            layout = ElementConnector.getFullLayoutForConfiguration(cll.getConfiguration(), this);
            cll.setParameterLayout(layout);
        } else {
            layout = cll.getParameterLayout();
            ((FrameLayout)layout.getParent()).removeView(layout);

        }
        builder.setView(layout);
        ElementConnector.addSaveBuilder(builder);
        setAlertDialog(builder.create());
        alertDialog.show();
    }

    private SavedToolkitApplication createSavedToolkitApplication(String appname, String description, boolean saveBTMac) {
        SavedToolkitApplication savedToolkitApplication = new SavedToolkitApplication(appname, saveBTMac);
        savedToolkitApplication.setBoards(Model.getInstance().getBoards());
        savedToolkitApplication.setConnectorConfigurationPoints(getConfigurationPointsList());
        savedToolkitApplication.setDescription(description);

        return savedToolkitApplication;
    }

    private List<List<ConfigurationPoint>> getConfigurationPointsList() {
        List<List<ConfigurationPoint>> confLists = new ArrayList<List<ConfigurationPoint>>();
        ViewGroup viewGroup = ((ViewGroup)findViewById(R.id.rule_container));
        for (int i = 0; i < viewGroup.getChildCount(); i++) {
            View view = viewGroup.getChildAt(i);
            if (view instanceof ConnectorLinearLayout) {
                confLists.add(getConfigurationPoints((ConnectorLinearLayout) view));
            }
        }
        return confLists;
    }

    public void greyOutOtherBoards(Board board) {
        List<ConfigurationLinearLayout> clls = getConfigurationLinearLayouts();
        for (ConfigurationLinearLayout cll : clls) {
            cll.getImage().setColorFilter(Color.LTGRAY, PorterDuff.Mode.MULTIPLY);
            cll.getImage().getBackground().setColorFilter(Color.LTGRAY, PorterDuff.Mode.MULTIPLY);
            cll.getTextView().setTypeface(null, Typeface.NORMAL);
            cll.getTextView().setTextColor(Color.DKGRAY);

            if (cll.getConfiguration().getBoard() == board) {
                cll.getImage().setColorFilter(null);
                cll.getImage().getBackground().setColorFilter(null);
                cll.getTextView().setTypeface(null, Typeface.BOLD);
                cll.getTextView().setTextColor(Color.BLACK);
            }
        }
    }

    private List<ConfigurationPoint> getConfigurationPoints(ConnectorLinearLayout connectorLinearLayout) {
        List<ConfigurationPoint> configurationPointList = new ArrayList<ConfigurationPoint>();
        for (ImageViewToImageViewArrow arrow : connectorLinearLayout.getConnectionArrows()) {
            if (arrow.getSourceLinearLayout() instanceof ConfigurationLinearLayout) {
                configurationPointList.add(createConfigurationPoint((ConfigurationLinearLayout) arrow.getSourceLinearLayout(), connectorLinearLayout));
            } else if (arrow.getTargetLinearLayout() instanceof ConfigurationLinearLayout) {
                configurationPointList.add(createConfigurationPoint((ConfigurationLinearLayout) arrow.getTargetLinearLayout(), connectorLinearLayout));
            }
        }
        return configurationPointList;
    }

    private ConfigurationPoint createConfigurationPoint(ConfigurationLinearLayout cll, ConnectorLinearLayout connectorLinearLayout) {
        Point point = cll.getDrawPoint();
        return new ConfigurationPoint(cll.getConfiguration().getId(), point.x, point.y, connectorLinearLayout.getId());
    }

    public void updateNavigationDrawerAdapter(){
        navigationDrawer.updateAdapter();
    }

    public void redrawRuleContainer(List<List<ConfigurationPoint>> points) {
        List<ConfigurationLinearLayout> clls = drawConfigurationLinearLayouts(points);
        Model.getInstance().clearAllRules();
        ConnectorIdGenerator.getInstance().flush();
        drawArrowsAndConnectors(clls, points);
        greyOutOtherBoards(Model.getInstance().getActiveBoard());
    }

    public List<ConfigurationLinearLayout> drawConfigurationLinearLayouts(List<List<ConfigurationPoint>> points) {
        ((ViewGroup) findViewById(R.id.rule_container)).removeAllViews();
        ConfigRulesOnDragListener configRulesOnDragListener = new ConfigRulesOnDragListener(this);
        List<ConfigurationLinearLayout> addedCLLs = new ArrayList<ConfigurationLinearLayout>();
        List<Integer> addedConfigurations = new ArrayList<Integer>();

        for (List<ConfigurationPoint> configurationPoints : points) {
            for(Board board : Model.getInstance().getBoards()) {
                for(Rule rule : board.getRules()) {
                    for(Configuration configuration : rule.getConfigurations()) {
                        for (ConfigurationPoint point : configurationPoints) {
                            if(point.getConfigurationId() == configuration.getId() && !addedConfigurations.contains(point.getConfigurationId())) {
                                addedCLLs.add(configRulesOnDragListener.addConfigurationToRuleContainer(
                                    configuration, new Point(point.getX(), point.getY()), false));
                                addedConfigurations.add(point.getConfigurationId());
                            }
                        }
                    }
                }
            }
        }

        return addedCLLs;
    }

    private static ConfigurationLinearLayout getMatchingCll(List<ConfigurationLinearLayout> clls, int configurationId) {
        for(ConfigurationLinearLayout cll : clls) {
            if(configurationId == cll.getConfiguration().getId()) {
                return cll;
            }
        }
        return null;
    }

    private void drawArrowsAndConnectors(List<ConfigurationLinearLayout> clls, List<List<ConfigurationPoint>> pointsList) {
        for (List<ConfigurationPoint> points : pointsList) {
            ConfigurationLinearLayout actionCLL = null;
            int actionCLLIndex = 0;
            ConfigurationLinearLayout conditionCLL = null;
            int conditionCLLIndex = 0;
            ConnectorLinearLayout connectorLinearLayout = null;
            for (ConfigurationPoint point : points) {
                ConfigurationLinearLayout cll = getMatchingCll(clls, point.getConfigurationId());
                if (conditionCLL == null && cll.getConfiguration() instanceof Condition) {
                    conditionCLL = cll;
                    conditionCLLIndex = points.indexOf(point);
                } else if (actionCLL == null && cll.getConfiguration() instanceof  Action){
                    actionCLL = cll;
                    actionCLLIndex = points.indexOf(point);
                }
                if (conditionCLL != null && actionCLL != null) {
                    connectorLinearLayout = initialConnection(conditionCLL, actionCLL, point.getConnectorId());
                    break;
                }
            }

            for (; actionCLLIndex < points.size(); actionCLLIndex++) {
                ConfigurationLinearLayout configurationLinearLayout = getMatchingCll(clls, points.get(actionCLLIndex).getConfigurationId());
                if (configurationLinearLayout.getConfiguration() instanceof Action) {
                    connectActionWithConnector(connectorLinearLayout, configurationLinearLayout);
                }
            }

            for (; conditionCLLIndex < points.size(); conditionCLLIndex++) {
                ConfigurationLinearLayout configurationLinearLayout = getMatchingCll(clls, points.get(conditionCLLIndex).getConfigurationId());
                if (configurationLinearLayout.getConfiguration() instanceof Condition) {
                    drawArrowBetweenConnectedLinearLayouts(configurationLinearLayout, connectorLinearLayout, R.drawable.line);
                    connectConditionWithConnector(configurationLinearLayout, connectorLinearLayout);
                }
            }
        }
    }

    public void browseAppStore() {
        ArrayList<String> list = new ArrayList<String>();
        for(Board board : Model.getInstance().getBoards()) {
            list.add(board.getHardwareSetup().getName());
        }
        Intent appStoreConfigIntent = new Intent(this, AppStoreConfiguration.class);
        appStoreConfigIntent.putStringArrayListExtra("query", list);
        appStoreConfigIntent.putExtra("exact", true);
        startActivity(appStoreConfigIntent);
    }

    public void saveApplicationToAppStore() {
        final EditText name = ElementCreator.createEditTextField(this, "", "Enter app name");
        LinearLayout nameLayout = new LinearLayout(this);
        nameLayout.addView(name);
        nameLayout.setLayoutParams(new LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT, 1f));
        final EditText description = ElementCreator.createEditTextField(this, "", "Enter app description");
        LinearLayout descriptionLayout = new LinearLayout(this);
        descriptionLayout.addView(description);
        descriptionLayout.setLayoutParams(new LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT, 1f));

        LinearLayout linearLayout = new LinearLayout(this);
        linearLayout.setOrientation(LinearLayout.VERTICAL);
        linearLayout.setLayoutParams(new LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT));
        linearLayout.addView(nameLayout);
        linearLayout.addView(descriptionLayout);

        AlertDialog.Builder builder = new AlertDialog.Builder(ConfigureRules.this);
        builder.setView(linearLayout);
        builder.setTitle("Chose App Name");
        builder.setPositiveButton(R.string.save, new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                String enteredName = name.getText().toString();
                String enteredDescription = description.getText().toString();
                if (enteredName != null && !enteredName.equals("") && enteredDescription != null && !enteredDescription.equalsIgnoreCase("")) {
                    save(enteredName, enteredDescription);
                    Toast.makeText(activity, "Your app was saved to the RAPT app store!", Toast.LENGTH_LONG).show();
                } else {
                    Toast.makeText(activity, "Your app was not saved: Please enter name and description", Toast.LENGTH_LONG).show();
                }
                dialog.dismiss();
            }
        });
        setAlertDialog(builder.create());
        alertDialog.show();
    }

    public void save(String name, String description) {
        SavedToolkitApplication application = createSavedToolkitApplication(name, description, false);
        AppServerConnection appServerConnection = new AppServerConnectionStub();
        appServerConnection.save(Serialization.serializeObject(application));
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if(requestCode == Constant.SINGLE_UPLOAD_REQUEST){
            if (resultCode == RESULT_OK) {
                initiateRulesInstallation(RulesInstaller.InstallType.Single);
            }

        } else if (requestCode == Constant.INTERCONNECTED_UPLOAD_ADVANCED_REQUEST) {
            if (resultCode == RESULT_OK) {
                if (easyUpload) {
                    initiateRulesInstallation(RulesInstaller.InstallType.Interconnected);
                } else {
                    showAdvancedInstallationDialog();
                }
            }
        } else if (requestCode == Constant.BOARD_MAPPING_UPLOAD_REQUEST) {
            finishActivity(Constant.BOARD_MAPPING_UPLOAD_REQUEST);
            if(resultCode == RESULT_OK) {
                initiateRulesInstallation(RulesInstaller.InstallType.Interconnected);
            }
        } else if (requestCode == Constant.MULTIPLE_EQUAL_UPLOAD_ADVANCE_REQUEST) {
            if(resultCode == RESULT_OK) {
                initiateRulesInstallation(RulesInstaller.InstallType.MultipleEqual);
            }
        } else if (requestCode == Constant.BOARD_MAPPING_MENU_REQUEST) {
            if(resultCode == RESULT_OK){
                navigationDrawer.updateAdapter();
            }
        } else if (requestCode == Constant.ENABLE_BT_REQUEST) {
            discoverDevices.initiateSearch();
        }
    }

    private void saveDraft() {
        Model.getInstance().setDraft(Serialization.serializeObject(createSavedToolkitApplication("draft", "", true)));
        Toast.makeText(this, "Draft saved", Toast.LENGTH_LONG).show();
    }

    private void loadDraft() {
        if(Model.getInstance().getDraft() != null) {
            SavedToolkitApplication draft =(SavedToolkitApplication)
                    Serialization.deserializeObject(Model.getInstance().getDraft());
            if(draft != null) {
                Model.updateModel(draft, this);
                updateNavigationDrawerAdapter();
                initiateBoardItems();
                redrawRuleContainer(draft.getConnectorConfigurationPoints());
            }
        } else {
            Toast.makeText(this, "No draft found", Toast.LENGTH_LONG).show();
        }
    }

    private void addSettingsIconInMenuBar() {
        try {
            ViewConfiguration config = ViewConfiguration.get(this);
            Field menuKeyField = ViewConfiguration.class
                    .getDeclaredField("sHasPermanentMenuKey");
            if (menuKeyField != null) {
                menuKeyField.setAccessible(true);
                menuKeyField.setBoolean(config, false);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if(alertDialog != null) {
            alertDialog.dismiss();
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        this.menu = menu;
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.action_bar, menu);
        setInitialMenuSettings(menu);
        menu.findItem(R.id.arrowMode).setChecked(true);
        menu.findItem(R.id.arrowMode).setIcon(R.drawable.ic_action_forward_pressed);
        return super.onCreateOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        if (actionBarDrawerToggle.onOptionsItemSelected(item)) {
            return true;
        }
        switch (item.getItemId()) {
            case R.id.installConfigurations:
                if (!isRulesValid()) {
                    Toast.makeText(this, "Limitation: This board can not read RFID tags and receive WiFi messages in the same app", Toast.LENGTH_LONG).show();
                } else if (validateMessageSettings()) {
                    installConfigurations();
                }
                return true;
            case R.id.boardName:
                changeBoardName(item);
                return true;
            case R.id.handMode:
                setMode(Mode.DRAG);
                toggleHandArrowIcons(item);
                return true;
            case R.id.arrowMode:
                setMode(Mode.CONNECT);
                toggleHandArrowIcons(item);
                return true;
            case R.id.appstore:
                showAppStoreDialog();
                return true;
            case  R.id.network_settings:
                Intent toNetworkSettings = new Intent(this, ConfigureNetworkSettings.class);
                startActivity(toNetworkSettings.putExtra("extra", "fromMenu"));
                return true;
            case R.id.draft_save:
                saveDraft();
                return true;
            case R.id.draft_load:
                loadDraft();
                return true;
            case R.id.boardMapping:
                Intent boardMapping = new Intent(this, BoardBluetoothMapping.class);
                boardMapping.putExtra("fromActivity", "menu");
                startActivityForResult(boardMapping, Constant.BOARD_MAPPING_MENU_REQUEST);
            default:
                return super.onOptionsItemSelected(item);
        }
    }
    private boolean validateMessageSettings(){
        List<ConfigurationLinearLayout> configurationLinearLayouts = getConfigurationLinearLayouts();
        for (ConfigurationLinearLayout configurationLinearLayout : configurationLinearLayouts) {
            Configuration configuration = configurationLinearLayout.getConfiguration();
            if (Model.getInstance().configurationInRules(configuration)) {
                if (configuration.getDeviceFunction().getName().equalsIgnoreCase("sendMessage")) {
                    for (int i = 0; i < configuration.getDeviceFunction().getParameters().size(); i++) {
                        if (configuration.getDeviceFunction().getParameters().get(i).getLabel().equalsIgnoreCase("IP address")){
                            if(!FormatCheck.validateIPAddress(configuration.getDeviceFunction().getParameters().get(i).getValueAsString())){
                                configureParameters(configurationLinearLayout);
                                Toast.makeText(this, "Please enter a valid IP address", Toast.LENGTH_SHORT).show();
                                return false;
                            }
                        }
                        if (configuration.getDeviceFunction().getParameters().get(i).getLabel().equalsIgnoreCase("Port")){
                            if(!FormatCheck.validatePort(configuration.getDeviceFunction().getParameters().get(i).getValueAsString())){
                                configureParameters(configurationLinearLayout);
                                Toast.makeText(this, "Please enter a valid port number", Toast.LENGTH_SHORT).show();
                                return false;
                            }
                        }
                    }
                } else if (configuration.getDeviceFunction().getName().equalsIgnoreCase("receiveMessage")) {
                    if (!FormatCheck.validateMessage(configuration.getDeviceFunction().getParameters().get(0).getValueAsString())) {
                        configureParameters(configurationLinearLayout);
                        Toast.makeText(this, "Please enter a message", Toast.LENGTH_SHORT).show();
                        return false;
                    }
                }
            }
        }
        return true;
    }

    private void showAppStoreDialog() {
        AlertDialog.Builder builder = new AlertDialog.Builder(ConfigureRules.this);
        builder.setTitle("RAPT App store options");
        LinearLayout linearLayout = new LinearLayout(this);
        linearLayout.setOrientation(LinearLayout.VERTICAL);
        Button save = new Button(this);
        save.setBackgroundResource(R.drawable.button_background_border_selector);
        save.setTextColor(getResources().getColorStateList(R.color.button_text_selector));
        save.setText("Publish app to RAPT app store");
        save.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                dismissDialog();
                saveApplicationToAppStore();
            }
        });
        Button browse = new Button(this);
        browse.setText("Browse RAPT app store");
        browse.setBackgroundResource(R.drawable.button_background_border_selector);
        save.setTextColor(getResources().getColorStateList(R.color.button_text_selector));
        browse.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                dismissDialog();
                browseAppStore();
            }
        });
        linearLayout.addView(save);
        linearLayout.addView(browse);
        builder.setView(linearLayout);
        setAlertDialog(builder.create());
        alertDialog.show();
    }

    public ConnectorLinearLayout drawArrowsAndConnectorLLBetweenConfigLLs(ConfigurationLinearLayout sourceLayout,
                                                         ConfigurationLinearLayout targetLayout) {
        ConnectorLinearLayout connectorLinearLayout = new ConnectorLinearLayout(this);
        ImageView imageView = new ImageView(this);
        imageView.setScaleType(ImageView.ScaleType.FIT_XY);
        imageView.setImageResource(R.drawable.connector_icon);
        final float scale = getResources().getDisplayMetrics().density;
        int pixels = (int) (connectorImageViewSize * scale + 0.5f);
        connectorLinearLayout.setLayoutParams(new FrameLayout.LayoutParams(pixels, pixels));
        connectorLinearLayout.setImage(imageView);
        Point location = getPointBetweenViews(sourceLayout, targetLayout);
        connectorLinearLayout.setX(location.x + (sourceLayout.getImage().getHeight() / 2) - connectorImageViewSize - 25);
        connectorLinearLayout.setY(location.y + (sourceLayout.getImage().getHeight() / 2) - connectorImageViewSize - 25);

        connectorLinearLayout.setOnLongClickListener(new ConnectedLinearLayoutOnLongClickListener(this));
        connectorLinearLayout.setOnTouchListener(new ConnectedLinearLayoutOnTouchListener(this));
        ((ViewGroup) findViewById(R.id.rule_container)).addView(connectorLinearLayout);

        drawArrowBetweenConnectedLinearLayouts(sourceLayout, connectorLinearLayout, R.drawable.line);
        drawArrowBetweenConnectedLinearLayouts(connectorLinearLayout, targetLayout, R.drawable.arrow);
        return connectorLinearLayout;
    }

    public ConnectorLinearLayout drawArrowsAndConnectorLLBetweenConfigLLs(ConfigurationLinearLayout sourceLayout,
                                                                          ConfigurationLinearLayout targetLayout, int connectorId) {
        ConnectorLinearLayout connectorLinearLayout = new ConnectorLinearLayout(this, connectorId);
        ImageView imageView = new ImageView(this);
        imageView.setScaleType(ImageView.ScaleType.FIT_XY);
        imageView.setImageResource(R.drawable.connector_icon);
        final float scale = getResources().getDisplayMetrics().density;
        int pixels = (int) (connectorImageViewSize * scale + 0.5f);
        connectorLinearLayout.setLayoutParams(new FrameLayout.LayoutParams(pixels, pixels));
        connectorLinearLayout.setImage(imageView);
        Point location = getPointBetweenViews(sourceLayout, targetLayout);
        connectorLinearLayout.setX(location.x + (sourceLayout.getImage().getHeight() / 2) - connectorImageViewSize);
        connectorLinearLayout.setY(location.y + (sourceLayout.getImage().getHeight() / 2) - connectorImageViewSize);

        connectorLinearLayout.setOnLongClickListener(new ConnectedLinearLayoutOnLongClickListener(this));
        connectorLinearLayout.setOnTouchListener(new ConnectedLinearLayoutOnTouchListener(this));
        ((ViewGroup) findViewById(R.id.rule_container)).addView(connectorLinearLayout);

        drawArrowBetweenConnectedLinearLayouts(sourceLayout, connectorLinearLayout, R.drawable.line);
        drawArrowBetweenConnectedLinearLayouts(connectorLinearLayout, targetLayout, R.drawable.arrow);
        return connectorLinearLayout;
    }

    public void drawArrowBetweenConnectedLinearLayouts(ConnectedLinearLayout sourceLayout, ConnectedLinearLayout targetLayout,
                                                       int resId) {
        ImageViewToImageViewArrow arrowIV =
                (ImageViewToImageViewArrow) getArrowIVBetweenPoints(sourceLayout.getArrowSourcePoint(),
                        targetLayout.getArrowSourcePoint(),
                        ArrowType.IMAGEVIEW_TO_IMAGEVIEW, resId);
        arrowIV.setSourceLinearLayout(sourceLayout);
        sourceLayout.addConnectionArrow(arrowIV);
        arrowIV.setTargetLinearLayout(targetLayout);
        targetLayout.addConnectionArrow(arrowIV);
        ((ViewGroup) findViewById(R.id.rule_container)).addView(arrowIV);
        sourceLayout.bringToFront();
        if (targetLayout instanceof ConnectorLinearLayout) {
            targetLayout.bringToFront();
        }
    }

    public void drawArrowBetweenConnectedLLAndPoint(ConnectedLinearLayout sourceLayout, Point target) {
        Point source = new Point((int)(sourceLayout.getX() + sourceLayout.getWidth() / 2),
                (int)(sourceLayout.getY() + sourceLayout.getHeight() / 2));
        ImageViewToPointArrow arrowIV =
                (ImageViewToPointArrow) getArrowIVBetweenPoints(source, target, ArrowType.IMAGEVIEW_TO_POINT, R.drawable.arrow);
        arrowIV.setSourceLinearLayout(sourceLayout);
        sourceLayout.setTouchArrow(arrowIV);
        ((ViewGroup) findViewById(R.id.rule_container)).addView(arrowIV);
    }

    private ImageView getArrowIVBetweenPoints(Point source, Point target, ArrowType type, int resId) {
        ImageView arrow = null;
        if (type == ArrowType.IMAGEVIEW_TO_IMAGEVIEW) {
            arrow = new ImageViewToImageViewArrow(this);
        } else if (type == ArrowType.IMAGEVIEW_TO_POINT) {
            arrow = new ImageViewToPointArrow(this);
        }

        float rotation = getRotation(source, target);
        arrow.setScaleType(ImageView.ScaleType.FIT_XY);
        arrow.setImageResource(resId);
        final float scale = getResources().getDisplayMetrics().density;
        int pixels = (int) (arrowViewSize * scale + 0.5f);
        arrow.setLayoutParams(new FrameLayout.LayoutParams(getLength(source, target), pixels));

        arrow.setX(source.x);
        arrow.setY(source.y - (pixels / 2));
        arrow.setPivotY(25);
        arrow.setRotation(rotation);
        return arrow;
    }

    private enum ArrowType {
        IMAGEVIEW_TO_POINT, IMAGEVIEW_TO_IMAGEVIEW
    }

    public static Point getPointBetweenViews(View view1, View view2) {
        Point point1 = new Point((int)view1.getX(), (int)view1.getY());
        Point point2 = new Point((int)view2.getX(), (int)view2.getY());

        int distanceX = (point1.x - point2.x) / 2;
        int distanceY = (point1.y - point2.y) / 2;

        return  new Point(point1.x - distanceX, point1.y - distanceY);
    }

    public static int getLength(Point source, Point target) {
        return (int)Math.sqrt(Math.pow(target.x - source.x, 2) + Math.pow(target.y - source.y, 2));
    }

    public static float getRotation(Point source, Point target) {
        float rotation = (float) Math.toDegrees(Math.atan2(target.y - source.y, target.x - source.x));

        if(rotation < 0){
            rotation += 360;
        }
        return rotation;
    }

    //a board can not have both a rfidread and a wifi receivemessage due to arduino uno serial port limitations
    private boolean isRulesValid() {
        for (Board board : Model.getInstance().getBoardsWithRules()) {
            if (board.rulesContainsDeviceFunction("rfidread") && board.rulesContainsDeviceFunction("receivemessage")) {
                return false;
            }
        }
        return true;
    }

    public void initiateRulesInstallation (RulesInstaller.InstallType installType) {
        boolean canContinueToInstallation = true;

        if(installType == RulesInstaller.InstallType.Single) {
            if(!FormatCheck.validateMacAdress(Model.getInstance().getActiveBoard().getBluetoothMacAddress())) {
                alertDialog.dismiss();
                Intent redirectToBluetoothPage = new Intent(getApplicationContext(), Bluetooth.class);
                redirectToBluetoothPage.putExtra("fromActivity", "ConfigureRules");
                startActivityForResult(redirectToBluetoothPage, Constant.SINGLE_UPLOAD_REQUEST);
                canContinueToInstallation = false;
            }

        } else if (installType == RulesInstaller.InstallType.Interconnected){
            for(Board board : Model.getInstance().getBoardsWithRules()) {
                if(!FormatCheck.validateMacAdress(board.getBluetoothMacAddress())) {
                    Intent mapBoardAndBluetoothDevice = new Intent(this, BoardBluetoothMapping.class);
                    mapBoardAndBluetoothDevice.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
                    mapBoardAndBluetoothDevice.putExtra("fromActivity", "upload");
                    activity.startActivityForResult(mapBoardAndBluetoothDevice, Constant.BOARD_MAPPING_UPLOAD_REQUEST);
                    canContinueToInstallation = false;
                }
            }

        } else if (installType == RulesInstaller.InstallType.MultipleEqual) {
            for(Board board : Model.getInstance().getBoards()) {
                if(board.getHardwareSetup() == Model.getInstance().getActiveBoard().getHardwareSetup()) {
                    if(!FormatCheck.validateMacAdress(board.getBluetoothMacAddress())) {
                        Intent mapBoardAndBluetoothDevice = new Intent(this, BoardBluetoothMapping.class);
                        mapBoardAndBluetoothDevice.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
                        mapBoardAndBluetoothDevice.putExtra("fromActivity", "upload");
                        activity.startActivityForResult(mapBoardAndBluetoothDevice, Constant.MULTIPLE_EQUAL_UPLOAD_ADVANCE_REQUEST);
                        canContinueToInstallation = false;
                    }
                }
            }
        }
        if(canContinueToInstallation) {
            InstallationProgressDialogHandler iHandler = new InstallationProgressDialogHandler(this, installType);
            iHandler.showInstallationProgressDialog();
            RulesInstaller installer = new RulesInstaller(this, installType, iHandler);
            installer.execute();
        }
    }

    public void toggleHandArrowIcons(MenuItem item){
        MenuItem hand = menu.findItem(R.id.handMode);
        MenuItem arrow = menu.findItem(R.id.arrowMode);
        if(item.getItemId() == R.id.handMode){
            item.setIcon(R.drawable.hand_mode_pressed2);
            item.setChecked(true);
            arrow.setChecked(false);
            arrow.setIcon(R.drawable.ic_action_forward);
        } else if(item.getItemId() == R.id.arrowMode){
            item.setIcon(R.drawable.ic_action_forward_pressed);
            item.setChecked(true);
            hand.setChecked(false);
            hand.setIcon(R.drawable.hand_mode);
        }
    }

    @Override
    protected void onSaveInstanceState(Bundle savedInstanceState) {
        savedInstanceState.putByteArray("configurationPointList",
                Serialization.serializeObject(getConfigurationPointsList()));
        savedInstanceState.putBoolean("easyUpload", easyUpload);
    }

    @Override
    protected void onRestoreInstanceState(Bundle savedInstanceState) {
        List<List<ConfigurationPoint>> configurationPointsList = (List<List<ConfigurationPoint>>)
                Serialization.deserializeObject(savedInstanceState.getByteArray("configurationPointList"));
        redrawRuleContainer(configurationPointsList);
        easyUpload = savedInstanceState.getBoolean("easyUpload");
    }

    //These methods is needed for the Navigation Drawer and needs to be in this class
    @Override
    protected void onPostCreate(Bundle savedInstanceState) {
        super.onPostCreate(savedInstanceState);
        if(actionBarDrawerToggle != null) {
            actionBarDrawerToggle.syncState();
        }
    }

    @Override
    public void onConfigurationChanged(android.content.res.Configuration newConfig) {
        super.onConfigurationChanged(newConfig);
        actionBarDrawerToggle.onConfigurationChanged(newConfig);
    }

    public void disableRuleContainerScroll() {
        ((LockableScrollView)findViewById(R.id.rule_container_scroll)).setScrollingEnabled(false);

    }

    public void enableRuleContainerScroll() {
        ((LockableScrollView)findViewById(R.id.rule_container_scroll)).setScrollingEnabled(true);
    }

    @Override
    public boolean onPrepareOptionsMenu(Menu menu){
        updateMenuBoardNameDisplay();
        return super.onPrepareOptionsMenu(menu);
    }

    @Override
    protected void onResume() {
        super.onResume();
    }

    public void removeConnectorLinearLayout(ConnectorLinearLayout connectorLinearLayout) {
        List<Rule> rules = connectorLinearLayout.getAllRules();
        for(Rule rule : rules){
            rule.getBoard().removeRule(rule);
        }
        Object[] arrows = connectorLinearLayout.getConnectionArrows().toArray();
        for (ImageViewToImageViewArrow arrow : connectorLinearLayout.getConnectionArrows()) {
            ((ViewGroup) findViewById(R.id.rule_container)).removeView(arrow);
        }
        for (Object arrow : arrows) {
            ((ImageViewToImageViewArrow)arrow).disconnectFromConnectedImageViews();
        }
        ((ViewGroup) findViewById(R.id.rule_container)).removeView(connectorLinearLayout);

    }

    public void initialConnection(ConfigurationLinearLayout conditionLL, ConfigurationLinearLayout actionLL) {
        ConnectorLinearLayout connectorLL;
        if(conditionLL.getConfiguration().getBoard() == actionLL.getConfiguration().getBoard()){
            connectorLL = drawArrowsAndConnectorLLBetweenConfigLLs(conditionLL, actionLL);
            Rule rule = createRuleOnSameBoard(conditionLL, actionLL);
            connectorLL.addRuleToBoardsOtherRule(rule);
        } else {
            if (actionLL.getConfiguration().getBoard().getWifiDevice() != null &&
                    conditionLL.getConfiguration().getBoard().getWifiDevice() != null) {
                connectorLL = drawArrowsAndConnectorLLBetweenConfigLLs(conditionLL, actionLL);
                List<Rule> rules = createRuleOnDifferentBoards(conditionLL, actionLL, connectorLL);
                connectorLL.addRuleToBoardsSendRule(rules.get(0));
                connectorLL.addRuleToBoardsOtherRule(rules.get(1));
            } else{
                Toast.makeText(this, connectionNotSupportedMessage, Toast.LENGTH_LONG).show();
            }
        }
    }

    public ConnectorLinearLayout initialConnection(ConfigurationLinearLayout conditionLL, ConfigurationLinearLayout actionLL, int connectorId) {
        ConnectorLinearLayout connectorLL = null;
        if(conditionLL.getConfiguration().getBoard() == actionLL.getConfiguration().getBoard()){
            connectorLL = drawArrowsAndConnectorLLBetweenConfigLLs(conditionLL, actionLL, connectorId);
            Rule rule = createRuleOnSameBoard(conditionLL, actionLL);
            connectorLL.addRuleToBoardsOtherRule(rule);
            return connectorLL;
        } else {
            if (actionLL.getConfiguration().getBoard().getWifiDevice() != null &&
                    conditionLL.getConfiguration().getBoard().getWifiDevice() != null) {
                connectorLL = drawArrowsAndConnectorLLBetweenConfigLLs(conditionLL, actionLL, connectorId);
                List<Rule> rules = createRuleOnDifferentBoards(conditionLL, actionLL, connectorLL);
                connectorLL.addRuleToBoardsSendRule(rules.get(0));
                connectorLL.addRuleToBoardsOtherRule(rules.get(1));
            } else{
                Toast.makeText(this, connectionNotSupportedMessage, Toast.LENGTH_LONG).show();
            }
        }
        return connectorLL;
    }

    private List<Rule> createRuleOnDifferentBoards(ConfigurationLinearLayout conditionLL, ConfigurationLinearLayout actionLL, ConnectorLinearLayout connectorLinearLayout){
        Rule actionRule = new Rule(actionLL.getConfiguration().getBoard());
        Rule conditionRule = new Rule(conditionLL.getConfiguration().getBoard());

        String automaticMessageValue = createAutomaticMessageValue(conditionLL.getConfiguration().getBoard(),
                actionLL.getConfiguration().getBoard(), connectorLinearLayout);
        Condition receiveMessage = createReceiveMessageFromBoard((Action) actionLL.getConfiguration(), automaticMessageValue);
        Action sendMessage = createSendMessageFromBoard((Condition) conditionLL.getConfiguration(), automaticMessageValue, (Action) actionLL.getConfiguration());

        conditionRule.addConfiguration(sendMessage);
        conditionRule.addConfiguration(conditionLL.getConfiguration());

        actionRule.addConfiguration(receiveMessage);
        actionRule.addConfiguration(actionLL.getConfiguration());

        List<Rule> rules = new ArrayList<Rule>();
        rules.add(conditionRule);
        rules.add(actionRule);
        return rules;
    }

    private Rule createRuleOnSameBoard(ConfigurationLinearLayout conditionLL, ConfigurationLinearLayout actionLL) {
        Rule newRule = new Rule(conditionLL.getConfiguration().getBoard());
        newRule.addConfiguration(actionLL.getConfiguration());
        newRule.addConfiguration(conditionLL.getConfiguration());
        return newRule;
    }

    private AutomaticReceiveMessage createReceiveMessageFromBoard(Action destinationAction, String automaticMessageValue){
        AutomaticReceiveMessage receiveMessage = new AutomaticReceiveMessage(destinationAction.getBoard());
        receiveMessage.setMessage(automaticMessageValue);
        receiveMessage.setDevice(destinationAction.getBoard().getWifiDevice());
        return receiveMessage;

    }

    private AutomaticSendMessage createSendMessageFromBoard(Condition sourceCondition, String automaticMessageValue, Action destinationAction){
        AutomaticSendMessage sendMessage = new AutomaticSendMessage(sourceCondition.getBoard());
        sendMessage.setIpAddress(destinationAction.getBoard().getWifiSettings().getIpAddress());
        sendMessage.setPort("2000");
        sendMessage.setMessage(automaticMessageValue);
        sendMessage.setDestinationBoard(destinationAction.getBoard());
        destinationAction.getBoard().getWifiSettings().addIpAddressChangeListener(sendMessage);
        sendMessage.setDevice(sourceCondition.getBoard().getWifiDevice());
        return sendMessage;
    }

    private String createAutomaticMessageValue(Board sourceBoard,Board targetBoard, ConnectorLinearLayout connectorLinearLayout) {
        return "AutomaticSendMessage-Source:"+ sourceBoard.getId()+ "-target:" + targetBoard.getId() + "-con:" +  connectorLinearLayout.getId();
    }


    public void connectConditionWithConnector(ConfigurationLinearLayout conditionLL, ConnectorLinearLayout connectorLL) {
        if (connectorLL.canConnect(conditionLL)) {
            addConditionToRules(conditionLL, connectorLL);
            drawArrowBetweenConnectedLinearLayouts(conditionLL, connectorLL, R.drawable.line);
        } else {
            Toast.makeText(this, connectionNotSupportedMessage, Toast.LENGTH_LONG).show();
        }
    }

    private void addConditionToRules(ConfigurationLinearLayout conditionLL, ConnectorLinearLayout connectorLinearLayout) {

        List<ConfigurationLinearLayout> actionLLs = connectorLinearLayout.getAllActionLinearLayouts();
        for (ConfigurationLinearLayout actionLL : actionLLs) {
            if (configurationsOnSameBoard(actionLL.getConfiguration(), conditionLL.getConfiguration())) {
                addConfigurationOnSameBoard(conditionLL, actionLL, connectorLinearLayout);
            }
            else {
                addConfigurationOnDifferentBoards(conditionLL, actionLL, connectorLinearLayout);
            }
        }
    }

    private void addActionToRules(ConfigurationLinearLayout actionLL, ConnectorLinearLayout connectorLinearLayout) {
        List<ConfigurationLinearLayout> conditionLLs = connectorLinearLayout.getAllConditionLinearLayouts();
        for (ConfigurationLinearLayout conditionLL : conditionLLs) {
            if (configurationsOnSameBoard(actionLL.getConfiguration(), conditionLL.getConfiguration())) {
                addConfigurationOnSameBoard(conditionLL, actionLL, connectorLinearLayout);
            }
            else {
                addConfigurationOnDifferentBoards(conditionLL, actionLL, connectorLinearLayout);
            }
        }
    }

    private void addConfigurationOnDifferentBoards(ConfigurationLinearLayout conditionLL, ConfigurationLinearLayout actionLL, ConnectorLinearLayout connectorLinearLayout) {
        String automaticMessageValue = createAutomaticMessageValue(conditionLL.getConfiguration().getBoard(),
                actionLL.getConfiguration().getBoard(), connectorLinearLayout);
        AutomaticReceiveMessage receiveMessage = createReceiveMessageFromBoard((Action) actionLL.getConfiguration(),automaticMessageValue);
        AutomaticSendMessage sendMessage = createSendMessageFromBoard((Condition) conditionLL.getConfiguration(), automaticMessageValue, (Action) actionLL.getConfiguration());
        Rule conditionRule = connectorLinearLayout.getBoardsSendRuleByBoard(conditionLL.getConfiguration().getBoard());
        Rule actionRule = connectorLinearLayout.getBoardsOtherRuleByBoard(actionLL.getConfiguration().getBoard());

        if(conditionRule == null){
            Rule newRule = new Rule(conditionLL.getConfiguration().getBoard());
            connectorLinearLayout.addRuleToBoardsSendRule(newRule);
        }
        if(actionRule == null){
            Rule newRule = new Rule(actionLL.getConfiguration().getBoard());
            connectorLinearLayout.addRuleToBoardsOtherRule(newRule);
        }

        if(!connectorLinearLayout.boardsSendRuleContainingSendMessage(conditionLL.getConfiguration().getBoard(), sendMessage)){
            connectorLinearLayout.getBoardsSendRuleByBoard(conditionLL.getConfiguration().getBoard()).addConfiguration(sendMessage);
        }
        if(!connectorLinearLayout.boardsOtherRuleContainingReceiveMessage(actionLL.getConfiguration().getBoard(), receiveMessage)){
            connectorLinearLayout.getBoardsOtherRuleByBoard(actionLL.getConfiguration().getBoard()).addConfiguration(receiveMessage);
        }

        connectorLinearLayout.getBoardsSendRuleByBoard(conditionLL.getConfiguration().getBoard()).addConfiguration(conditionLL.getConfiguration());
        connectorLinearLayout.getBoardsOtherRuleByBoard(actionLL.getConfiguration().getBoard()).addConfiguration(actionLL.getConfiguration());
    }



    private void addConfigurationOnSameBoard(ConfigurationLinearLayout conditionLL, ConfigurationLinearLayout actionLL, ConnectorLinearLayout connectorLinearLayout){
        Rule rule = connectorLinearLayout.getBoardsOtherRuleByBoard(conditionLL.getConfiguration().getBoard());

        if(rule == null){
            Rule newRule = new Rule(conditionLL.getConfiguration().getBoard());
            newRule.addConfiguration(conditionLL.getConfiguration());
            newRule.addConfiguration(actionLL.getConfiguration());
            connectorLinearLayout.addRuleToBoardsOtherRule(newRule);
        } else{
            rule.addConfiguration(conditionLL.getConfiguration());
            rule.addConfiguration(actionLL.getConfiguration());
        }
    }

    private boolean configurationsOnSameBoard(Configuration configuration1, Configuration configuration2){
        return configuration1.getBoard() == configuration2.getBoard();
    }

    public void connectActionWithConnector(ConnectorLinearLayout cll, ConfigurationLinearLayout actionLL) {
        if (cll.canConnect(actionLL)) {
            addActionToRules(actionLL, cll);
            drawArrowBetweenConnectedLinearLayouts(cll, actionLL, R.drawable.arrow);
        } else {
            Toast.makeText(this, connectionNotSupportedMessage, Toast.LENGTH_LONG).show();
        }
    }
}