/*
 * Licensed to UbiCollab.org under one or more contributor
 * license agreements.  See the NOTICE file distributed
 * with this work for additional information regarding
 * copyright ownership. UbiCollab.org licenses this file
 * to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package no.ntnu.android.toolkit.activities;

import android.app.Activity;
import android.content.Intent;
import android.content.pm.ActivityInfo;
import android.os.Bundle;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.ListView;
import android.widget.Toast;
import no.ntnu.android.toolkit.graphic.adapter.BoardTypeAdapter;
import no.ntnu.android.toolkit.model.HardwareSetup;
import no.ntnu.android.toolkit.model.Model;
import no.ntnu.android.toolkit.reader.BoardTypeReader;

public class SelectBoardType extends Activity {

    private BoardTypeAdapter boardTypeAdapter;
    private BoardTypeReader boardTypeReader;
    private Button configureButton;
    private int selectedNumber = 0;

    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.boardtypeselection);
        getActionBar().setTitle("Select board type to configure");
        getActionBar().setDisplayHomeAsUpEnabled(true);
        setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
        configureButton = (Button)findViewById(R.id.buttonSelectedBoardTypes);
        configureButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Model.getInstance().clearAllBoards();
                if(addSelectedBoardTypeAsBoards()) {
                    BoardTypeReader.addDeviceFunctionsToBoards(SelectBoardType.this);
                    showSelectedModel();
                } else{
                    Toast.makeText(getApplicationContext(), "You need to select at least one board", Toast.LENGTH_SHORT).show();
                }
            }
        });

        initialiseAdapter();
    }

    public void addNumberOfSelectedOnButton(int addToNumber) {
        selectedNumber = selectedNumber + addToNumber;
        configureButton.setText(getString(R.string.createApp) + " (" + selectedNumber + ")");
    }

    public void initialiseAdapter(){
        final ListView listview = (ListView) findViewById(R.id.listView);
        listview.setChoiceMode(ListView.CHOICE_MODE_MULTIPLE);
        listview.setItemsCanFocus(false);
        boardTypeReader = new BoardTypeReader(this);

        boardTypeAdapter = new BoardTypeAdapter(this,
                android.R.layout.simple_list_item_multiple_choice);

        listview.setAdapter(boardTypeAdapter);
        updateAdapter();
    }

    public void updateAdapter() {
        configureButton.setText(getString(R.string.createApp));
        selectedNumber = 0;
        boardTypeAdapter.clear();
        boardTypeAdapter.addAll(boardTypeReader.getDevices());
        boardTypeAdapter.notifyDataSetChanged();
    }

    public void showSelectedModel() {
        Intent toConfigureRules = new Intent(this, ConfigureRules.class);
        startActivity(toConfigureRules);
    }

    @Override
    public void onBackPressed() {
        super.onBackPressed();
    }

    private boolean addSelectedBoardTypeAsBoards() {
        boolean added = false;
        for(int i = 0; i < boardTypeAdapter.getCount(); i++) {
            HardwareSetup hardwareSetup = boardTypeAdapter.getHardwareSetup(i);
            int number = boardTypeAdapter.getNumberOf(i);
            for(int j = 0; j < number; j++) {
                Model.getInstance().createAddBoard(hardwareSetup);
                added = true;
            }
        }
        return added;
    }



    @Override
    public void onResume() {
        super.onResume();
        updateAdapter();
    }


    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        if(item.getItemId() == android.R.id.home) {
            onBackPressed();
        }
        return super.onOptionsItemSelected(item);
    }
}