/*
 * Licensed to UbiCollab.org under one or more contributor
 * license agreements.  See the NOTICE file distributed
 * with this work for additional information regarding
 * copyright ownership. UbiCollab.org licenses this file
 * to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package no.ntnu.android.toolkit.appstore;

import no.ntnu.android.toolkit.model.Board;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

public class SavedToolkitApplication implements Serializable {

    private static final long serialVersionUID = 7L;

    private String name = "No_name";
    private HashMap<Board, String> boardBTMacAddresses;
    private String description ="";
    private List<Board> boards;
    private List<List<ConfigurationPoint>> connectorConfigurationPoints;
    private boolean saveBTMac;

    public SavedToolkitApplication(String name, boolean saveBTMac) {
        if (name != null) {
            this.name = name;
        }
        this.saveBTMac = saveBTMac;
        boards = new ArrayList<Board>();
        boardBTMacAddresses = new HashMap<Board, String>();
        connectorConfigurationPoints = new ArrayList<List<ConfigurationPoint>>();
    }

    public List<Board> getBoards() {
        return boards;
    }

    public void setBoards(List<Board> boards) {
        this.boards = boards;
        if (saveBTMac) {
            for (Board board : boards) {
                boardBTMacAddresses.put(board, board.getBluetoothMacAddress());
            }
        }
    }

    public void addBoard(Board board) {
        if(!boards.contains(board)) {
            boards.add(board);
            if (saveBTMac) {
                boardBTMacAddresses.put(board, board.getBluetoothMacAddress());
            }
        }
    }

    public List<List<ConfigurationPoint>> getConnectorConfigurationPoints() {
        return connectorConfigurationPoints;
    }

    public void setConnectorConfigurationPoints(List<List<ConfigurationPoint>> connectorConfigurationPoints) {
        this.connectorConfigurationPoints = connectorConfigurationPoints;
    }

    public String getName() {
        return name;
    }

    public String toString() {
        return name;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public boolean isSaveBTMac() {
        return saveBTMac;
    }

    public HashMap<Board, String> getBoardBTMacAddresses() {
        return boardBTMacAddresses;
    }
}