/*
 * Licensed to UbiCollab.org under one or more contributor
 * license agreements.  See the NOTICE file distributed
 * with this work for additional information regarding
 * copyright ownership. UbiCollab.org licenses this file
 * to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package no.ntnu.android.toolkit.bluetooth;

import android.app.ProgressDialog;
import android.content.DialogInterface;
import no.ntnu.android.toolkit.activities.ConfigureRules;

public class DiscoverDevicesProgressDialogHandler {

    private ConfigureRules configureRules;
    private ProgressDialog progressDialog;

    public enum EventType{
        DEVICE_ADDED
    }

    public DiscoverDevicesProgressDialogHandler(ConfigureRules activity){
        this.configureRules = activity;
    }

    public ProgressDialog createSearchDialog(){
        progressDialog = new ProgressDialog(configureRules);
        progressDialog.setTitle("Searching for nearby boards");
        progressDialog.setCancelable(false);
        progressDialog.setMessage("Searching...");
        progressDialog.setProgressStyle(ProgressDialog.STYLE_SPINNER);
        progressDialog.setButton(DialogInterface.BUTTON_NEGATIVE, "Cancel search", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialogInterface, int i) {
                configureRules.onBackPressed();
            }
        });
        return progressDialog;
    }

    public void deviceDiscoveredChange(EventType eventType){
        switch (eventType){
            case DEVICE_ADDED:
                configureRules.updateNavigationDrawerAdapter();
                progressDialog.dismiss();
                break;
            default:
                break;
        }
    }

}
