/*
 * Licensed to UbiCollab.org under one or more contributor
 * license agreements.  See the NOTICE file distributed
 * with this work for additional information regarding
 * copyright ownership. UbiCollab.org licenses this file
 * to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package no.ntnu.android.toolkit.graphic;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;

public class ConnectorIdGenerator {

    private AtomicInteger atomicInteger;
    private static ConnectorIdGenerator connectorIdGenerator;
    private List<Integer> usedIds = new ArrayList<Integer>();

    private ConnectorIdGenerator(){
        atomicInteger = new AtomicInteger(1);
    }

    public static ConnectorIdGenerator getInstance() {
        if (connectorIdGenerator == null) {
            connectorIdGenerator = new ConnectorIdGenerator();
        }
        return connectorIdGenerator;
    }

    public int getNext() {
        int newId = atomicInteger.incrementAndGet();
        usedIds.add(newId);
        return newId;
    }

    public void registerIdUsage(int id) {
        if (usedIds.contains(id)) {
            throw new IllegalStateException("Two connectors can not hold the same id: " + id);
        } else {
            usedIds.add(id);
            if (atomicInteger.get() <= id) {
                atomicInteger.set(id + 1);
            }
        }
    }

    public void flush() {
        atomicInteger = new AtomicInteger(1);
        usedIds = new ArrayList<Integer>();
    }
}
