/*
 * Licensed to UbiCollab.org under one or more contributor
 * license agreements.  See the NOTICE file distributed
 * with this work for additional information regarding
 * copyright ownership. UbiCollab.org licenses this file
 * to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package no.ntnu.android.toolkit.graphic;

import android.content.Context;
import android.content.Intent;
import android.support.v4.app.ActionBarDrawerToggle;
import android.support.v4.widget.DrawerLayout;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ListView;
import android.widget.TextView;
import no.ntnu.android.toolkit.activities.*;
import no.ntnu.android.toolkit.graphic.adapter.NavigationDrawerAdapter;
import no.ntnu.android.toolkit.graphic.adapter.NavigationDrawerMenuListAdapter;
import no.ntnu.android.toolkit.graphic.listener.NavigationDrawerListener;
import no.ntnu.android.toolkit.model.Board;
import no.ntnu.android.toolkit.model.Model;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

public class NavigationDrawer {
    private ConfigureRules configureRules;
    private NavigationDrawerAdapter<Board> adapter;
    private ActionBarDrawerToggle actionBarDrawerToggle;
    private ListView boardList;

    public NavigationDrawer(ConfigureRules configureRules){
        this.configureRules = configureRules;
}

    public ActionBarDrawerToggle getActionBarDrawerToggle(){
        return actionBarDrawerToggle;
    }

    public void initDrawerLayout(){
        DrawerLayout drawerLayout = (DrawerLayout) configureRules.findViewById(R.id.drawer_layout);

        actionBarDrawerToggle = new ActionBarDrawerToggle(
                configureRules, drawerLayout, R.drawable.ic_drawer, R.string.drawerOpen, R.string.drawerClose);
        drawerLayout.setDrawerListener(actionBarDrawerToggle);
        configureRules.getActionBar().setDisplayHomeAsUpEnabled(true);
        configureRules.getActionBar().setHomeButtonEnabled(true);
        initShortcuts();
        adapter = new NavigationDrawerAdapter<Board>(configureRules, android.R.layout.simple_list_item_1);
        boardList.setAdapter(adapter);
        adapter.notifyDataSetChanged();
        boardList.setOnItemClickListener(new NavigationDrawerListener(configureRules, this));
        drawerLayout.openDrawer(Gravity.LEFT);
        updateAdapter();
    }

    private void styleNavigationDrawer(ListView drawerList){
        drawerList.setBackgroundResource(android.R.color.background_light);
    }

    private void initShortcuts(){

        boardList = (ListView)configureRules.findViewById(R.id.left_drawer);
        styleNavigationDrawer(boardList);
        View footerView = ((LayoutInflater) configureRules.getSystemService(Context.LAYOUT_INFLATER_SERVICE)).inflate(R.layout.navigation_drawer_menu_items, null, false);
        boardList.setDividerHeight(1);


        ListView menuItemsList = (ListView)footerView.findViewById(R.id.menuItemsList);
        menuItemsList.setFooterDividersEnabled(true);
        menuItemsList.setHeaderDividersEnabled(true);
        menuItemsList.setDividerHeight(10);
        List<String> menuItems = new ArrayList<String>();
        menuItems.add(configureRules.getString(R.string.mainMenu));
        menuItems.add(configureRules.getString(R.string.createApp));
        menuItems.add(configureRules.getString(R.string.advancedMode));
        menuItems.add(configureRules.getString(R.string.appStore));
        NavigationDrawerMenuListAdapter adapter1 = new NavigationDrawerMenuListAdapter(configureRules.getApplicationContext(), android.R.layout.simple_list_item_activated_1, menuItems);

        menuItemsList.setAdapter(adapter1);
        menuItemsList.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> adapterView, View view, int i, long l) {
                String text =((TextView)view.findViewById(R.id.menuItemText)).getText().toString();
                Intent intent = null;

                if(text.equalsIgnoreCase(configureRules.getString(R.string.createApp))){
                    configureRules.clearRuleContainer();
                    configureRules.startBluetoothSearch();

                } else if (text.equalsIgnoreCase(configureRules.getString(R.string.advancedMode))) {
                    intent = new Intent(configureRules, SelectBoardType.class);

                } else if (text.equalsIgnoreCase(configureRules.getString(R.string.appStore))) {
                    intent = new Intent(configureRules, AppStore.class);

                } else if (text.equalsIgnoreCase(configureRules.getString(R.string.mainMenu))) {
                    intent = new Intent(configureRules, MyActivity.class);
                    intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);

                } else {
                    return;
                }

                if(intent != null) {
                    configureRules.startActivity(intent);
                }
            }
        });

        adapter1.notifyDataSetChanged();
        boardList.addFooterView(footerView);

    }

    public void updateAdapter() {
        adapter.clear();
        List<String> namedBoard = new LinkedList<String>();
        for (Board board : Model.getInstance().getBoards()) {
            if (namedBoard.contains(board.getHardwareSetup().getName())) continue;
            int i = 1;
            for (Board board2 : Model.getInstance().getBoards()) {
                if (board != board2) {
                    if (board.getHardwareSetup().getName().equals(board2.getHardwareSetup().getName())) {
                        if (!board2.isNameChangedByUser()) {
                            board2.setName(board2.getHardwareSetup().getName() + "(" + i + ")");
                        }
                        i++;
                    }
                }
            }
            namedBoard.add(board.getHardwareSetup().getName());
        }

        adapter.addAll(Model.getInstance().getBoards());
        adapter.notifyDataSetChanged();
    }
}