/*
 * Licensed to UbiCollab.org under one or more contributor
 * license agreements.  See the NOTICE file distributed
 * with this work for additional information regarding
 * copyright ownership. UbiCollab.org licenses this file
 * to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package no.ntnu.android.toolkit.graphic.adapter;

import android.app.Activity;
import android.bluetooth.BluetoothDevice;
import android.content.Context;
import android.content.Intent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.Spinner;
import android.widget.TextView;
import no.ntnu.android.toolkit.activities.R;
import no.ntnu.android.toolkit.model.Board;
import no.ntnu.android.toolkit.model.HardwareSetup;
import no.ntnu.android.toolkit.model.Model;


public class BoardBluetoothAdapter extends ArrayAdapter<BoardBluetoothAdapter.BoardBluetoothListItem>{

    private Activity activity;
    private Intent intent;
    public BoardBluetoothAdapter(Context context, int textViewResourceId, Intent intent) {
        super(context, textViewResourceId);
        this.activity = (Activity)context;
        this.intent = intent;
    }

    @Override
    public View getView(int position, View convertView, ViewGroup parent) {
        View row = convertView;

        if (row == null) {
            LayoutInflater layoutInflater = (LayoutInflater)
                    getContext().getSystemService(Context.LAYOUT_INFLATER_SERVICE);
            row = layoutInflater.inflate(R.layout.board_bluetooth_list_item, null);
        }

        TextView textView = (TextView)row.findViewById(R.id.mappingBoard_bluetoothDevice);
        textView.setText(getItem(position).getBluetoothDeviceName());

        TextView macAddress = (TextView)row.findViewById(R.id.mappingBoard_bluetoothDeviceMacAddress);
        macAddress.setText(getItem(position).getBluetoothDeviceMac());


        Spinner spinner = (Spinner)row.findViewById(R.id.mappingBoard_boardSpinner);
        ArrayAdapter<Board> adapter = new ArrayAdapter<Board>(activity, android.R.layout.simple_list_item_1);
        spinner.setAdapter(adapter);
        Board emtpyBoard = new Board(new HardwareSetup());
        emtpyBoard.setName("Select board");
        emtpyBoard.setBluetoothMacAddress(null);
        adapter.add(emtpyBoard);
        for (Board board : Model.getInstance().getBoards()) {
            adapter.add(board);
        }
        adapter.notifyDataSetChanged();
        return row;
    }

    public void add(android.bluetooth.BluetoothDevice bluetoothDevice) {
        BoardBluetoothListItem boardBluetoothListItem = new BoardBluetoothListItem(bluetoothDevice);
        super.add(boardBluetoothListItem);
    }

    public class BoardBluetoothListItem {
        private BluetoothDevice bluetoothDevice;

        public BoardBluetoothListItem(BluetoothDevice bluetoothDevice) {
            this.bluetoothDevice = bluetoothDevice;
        }
        public String getBluetoothDeviceName(){
            return bluetoothDevice.getName();
        }
        public String getBluetoothDeviceMac(){
            return bluetoothDevice.getAddress();
        }

    }
}
