/*
 * Licensed to UbiCollab.org under one or more contributor
 * license agreements.  See the NOTICE file distributed
 * with this work for additional information regarding
 * copyright ownership. UbiCollab.org licenses this file
 * to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package no.ntnu.android.toolkit.graphic.adapter;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.NumberPicker;
import android.widget.TextView;
import no.ntnu.android.toolkit.activities.R;
import no.ntnu.android.toolkit.activities.SelectBoardType;
import no.ntnu.android.toolkit.model.HardwareSetup;

import java.util.List;

public class BoardTypeAdapter extends ArrayAdapter<BoardTypeAdapter.BoardTypeAndNumber> {

    public BoardTypeAdapter(Context context, int textViewResourceId) {
        super(context, textViewResourceId);
    }

    @Override
    public View getView(final int position, View convertView, ViewGroup parent) {
        View row = convertView;

        if (row == null) {
            LayoutInflater layoutInflater = (LayoutInflater)
                    getContext().getSystemService(Context.LAYOUT_INFLATER_SERVICE);
            row = layoutInflater.inflate(R.layout.boardtype_list_item, null);
        }

        final NumberPicker numberPicker = (NumberPicker)row.findViewById(R.id.numberpickerBoardType);
        numberPicker.setMaxValue(100);
        numberPicker.setMinValue(0);
        numberPicker.setValue(getItem(position).getNumber());
        numberPicker.setOnValueChangedListener(new NumberPicker.OnValueChangeListener() {
            @Override
            public void onValueChange(NumberPicker picker, int oldVal, int newVal) {
                getItem(position).setNumber(numberPicker.getValue());
                if(getContext() instanceof SelectBoardType) {
                    ((SelectBoardType)getContext()).addNumberOfSelectedOnButton(newVal-oldVal);
                }
            }
        });

        TextView textView = (TextView)row.findViewById(R.id.textViewNameBoardType);
        textView.setText(getItem(position).getHardwareSetup().getName());
        textView.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                int newValue = numberPicker.getValue() + 1;
                numberPicker.setValue(newValue);
                addObject(newValue, position);
                adjustForOverMaxValue(position, newValue, numberPicker.getMaxValue());
            }
        });

        TextView descriptionView = (TextView) row.findViewById(R.id.textSelectBoardTypeDescription);
        descriptionView.setText(getItem(position).getHardwareSetup().getDescription());
        descriptionView.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                int newValue = numberPicker.getValue() + 1;
                numberPicker.setValue(newValue);
                addObject(newValue, position);
                adjustForOverMaxValue(position, newValue, numberPicker.getMaxValue());
            }
        });

        return row;
    }

    private void addObject(int newValue, int position) {
        getItem(position).setNumber(newValue);
        if(getContext() instanceof SelectBoardType) {
            ((SelectBoardType)getContext()).addNumberOfSelectedOnButton(1);
        }
    }

    private void adjustForOverMaxValue(int position, int value, int max) {
        if(value > max) {
            int newValue = -(max+(value-max));
            getItem(position).setNumber(newValue);
            if (getContext() instanceof SelectBoardType) {
                ((SelectBoardType)getContext()).addNumberOfSelectedOnButton(newValue);
            }
        }
    }

    public void addAll(List<HardwareSetup> hardwareSetups) {
        for(HardwareSetup hardwareSetup : hardwareSetups) {
            add(hardwareSetup);
        }
    }

    public void add(HardwareSetup hardwareSetup) {
        super.add(new BoardTypeAndNumber(hardwareSetup));
    }

    public int getNumberOf(int position) {
        return super.getItem(position).getNumber();
    }

    public HardwareSetup getHardwareSetup(int position) {
        return super.getItem(position).getHardwareSetup();
    }

    class BoardTypeAndNumber {
        private HardwareSetup hardwareSetup;
        private int number;

        public BoardTypeAndNumber(HardwareSetup hardwareSetup) {
            this.hardwareSetup = hardwareSetup;
            this.number = 0;
        }

        public HardwareSetup getHardwareSetup() {
            return hardwareSetup;
        }

        public void setNumber(int number) {
            this.number = number;
        }

        public int getNumber() {
            return number;
        }
    }
}