/*
 * Licensed to UbiCollab.org under one or more contributor
 * license agreements.  See the NOTICE file distributed
 * with this work for additional information regarding
 * copyright ownership. UbiCollab.org licenses this file
 * to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package no.ntnu.android.toolkit.graphic.adapter;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.EditText;
import android.widget.TextView;
import no.ntnu.android.toolkit.activities.R;
import no.ntnu.android.toolkit.model.Board;
import no.ntnu.android.toolkit.model.Model;

import java.util.List;

public class IPSettingsAdapter extends ArrayAdapter<IPSettingsAdapter.BoardNameAndIpTextField>{


    public IPSettingsAdapter(Context context, int textViewResourceId) {
        super(context, textViewResourceId);
    }

    @Override
    public View getView(final int position, View convertView, ViewGroup parent) {
        View row = convertView;

        if (row == null) {
            LayoutInflater layoutInflater = (LayoutInflater)
                    getContext().getSystemService(Context.LAYOUT_INFLATER_SERVICE);
            row = layoutInflater.inflate(R.layout.board_ip_list_item, null);
        }
        final Board board = super.getItem(position).getBoard();

        EditText editTextBoardIp = null;
        if (row.findViewById(R.id.txtIPAddress) != null) {
            editTextBoardIp =  (EditText)row.findViewById(R.id.txtIPAddress);
        } else {
            for (Board board1 : Model.getInstance().getBoards()) {
                if (row.findViewById(board1.getId()) != null) {
                    editTextBoardIp = (EditText)row.findViewById(board1.getId());
                }
            }
        }
        editTextBoardIp.setId(board.getId());
        editTextBoardIp.setTag(board.getId());
        editTextBoardIp.setEnabled(false);
        if (board.getWifiDevice() != null) {
            if (board.getWifiSettings() != null) {
                if (board.getWifiSettings().getIpAddress() != null) {
                    editTextBoardIp.setText(board.getWifiSettings().getIpAddress());
                }
            }
        }

        TextView textView =(TextView)row.findViewById(R.id.boardNameText);
        textView.setText(board.getName());
        return row;
    }

    public void add(Board board) {
        super.add(new BoardNameAndIpTextField(board));
    }
    public void addAll(List<Board> boards) {
        for(Board board : boards) {
            add(board);
        }
    }

    public class BoardNameAndIpTextField {
        private Board board;

        public BoardNameAndIpTextField(Board board) {
            this.board = board;
        }

        public Board getBoard() {
            return board;
        }

    }
}