/*
 * Licensed to UbiCollab.org under one or more contributor
 * license agreements.  See the NOTICE file distributed
 * with this work for additional information regarding
 * copyright ownership. UbiCollab.org licenses this file
 * to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package no.ntnu.android.toolkit.graphic.adapter;

import android.content.res.Resources;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import no.ntnu.android.toolkit.activities.R;
import no.ntnu.android.toolkit.activities.Tutorial;

public class TutorialImage extends Fragment{

    public static String DRAWABLE_ITEM;

    public static TutorialImage newInstance(String drawableItem) {
        TutorialImage tutorialImage = new TutorialImage();
        Bundle bundle = new Bundle(1);
        bundle.putString(DRAWABLE_ITEM, drawableItem);
        tutorialImage.setArguments(bundle);
        return tutorialImage;
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        String drawableName = getArguments().getString(DRAWABLE_ITEM);
        View view = inflater.inflate(R.layout.tutorial_fragment, container, false);
        ImageView imageView = (ImageView)view.findViewById(R.id.tutorialImageView);
        int drawableId = getPictureId(drawableName);
        imageView.setImageBitmap(decodeSampledBitmapFromResource(getResources(), drawableId, 300, 500));
        return view;
    }

    private int getPictureId(String drawableName) {
        int drawableId = getResources().getIdentifier(drawableName.toLowerCase(), "drawable", Tutorial.PACKAGE_NAME);
        if (drawableId > 0) {
            return drawableId;
        }
        return 0;
    }
    public static Bitmap decodeSampledBitmapFromResource(Resources res, int resId,
                                                         int reqWidth, int reqHeight) {
        final BitmapFactory.Options bitmapFactoryOptions = new BitmapFactory.Options();
        bitmapFactoryOptions.inJustDecodeBounds = true;
        BitmapFactory.decodeResource(res, resId, bitmapFactoryOptions);
        bitmapFactoryOptions.inSampleSize = calculateBitmapSize(bitmapFactoryOptions, reqWidth, reqHeight);
        bitmapFactoryOptions.inJustDecodeBounds = false;
        return BitmapFactory.decodeResource(res, resId, bitmapFactoryOptions);
    }

    public static int calculateBitmapSize(
            BitmapFactory.Options bitmapFactoryOptions, int requiredWidth, int requiredHeight) {
        final int height = bitmapFactoryOptions.outHeight;
        final int width = bitmapFactoryOptions.outWidth;
        int bmSize = 1;

        if (height > requiredHeight || width > requiredWidth) {
            final int heightDivied = height / 2;
            final int widthDevided = width / 2;
            while ((heightDivied / bmSize) > requiredHeight
                    && (widthDevided / bmSize) > requiredWidth) {
                bmSize *= 2;
            }
        }
        return bmSize;
    }
}
