/*
 * Licensed to UbiCollab.org under one or more contributor
 * license agreements.  See the NOTICE file distributed
 * with this work for additional information regarding
 * copyright ownership. UbiCollab.org licenses this file
 * to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package no.ntnu.android.toolkit.graphic.listener;

import android.graphics.Color;
import android.graphics.Point;
import android.graphics.Typeface;
import android.util.TypedValue;
import android.view.*;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;
import no.ntnu.android.toolkit.activities.ConfigureRules;
import no.ntnu.android.toolkit.activities.R;
import no.ntnu.android.toolkit.graphic.ElementConnector;
import no.ntnu.android.toolkit.graphic.view.ConfigurationLinearLayout;
import no.ntnu.android.toolkit.graphic.view.ConnectedLinearLayout;
import no.ntnu.android.toolkit.graphic.view.ConnectorLinearLayout;
import no.ntnu.android.toolkit.graphic.view.ImageViewToImageViewArrow;
import no.ntnu.android.toolkit.model.*;
import no.ntnu.android.toolkit.tool.Search;

import java.util.List;

public class ConfigRulesOnDragListener implements View.OnDragListener {

    private ConfigureRules configureRules;

    public ConfigRulesOnDragListener(ConfigureRules configureRules) {
        this.configureRules = configureRules;
    }

    @Override
    public boolean onDrag(View v, DragEvent event) {
        if (event == null || event.getLocalState() == null || ((View)event.getLocalState()).getParent() == null) {
            return true;
        }
        int sourceViewId = ((View)(((View)event.getLocalState()).getParent())).getId();
        if (event.getAction() == DragEvent.ACTION_DROP) {
            if ((sourceViewId == R.id.input_container || sourceViewId == R.id.output_container) &&
                    v.getId() == R.id.rule_container) {
                return handleDragFromIOContainerToRuleContainer(v, event);

            } else if (sourceViewId == R.id.rule_container && v.getId() == R.id.rule_container) {
                return handleDragWithinRuleContainer(v, event);
            }
        } else if (event.getAction() == DragEvent.ACTION_DRAG_EXITED) {
            if (sourceViewId == R.id.rule_container && v.getId() == R.id.rule_container) {
                return handleDragFromRuleContainerToIOContainer(v, event);
            }
        }
        else if (event.getAction() == DragEvent.ACTION_DRAG_LOCATION) {
            if (sourceViewId == R.id.rule_container && v.getId() == R.id.rule_container) {
                return handleDragWithinRuleContainer(v, event);
            }
        }
        return true;
    }

    private boolean handleDragFromRuleContainerToIOContainer(View v, DragEvent event) {
        switch (event.getAction()) {
            case DragEvent.ACTION_DRAG_EXITED:
                configureRules.enableRuleContainerScroll();
                ConnectedLinearLayout draggedCLL = (ConnectedLinearLayout) event.getLocalState();
                if (draggedCLL instanceof ConnectorLinearLayout) {
                    configureRules.removeConnectorLinearLayout((ConnectorLinearLayout) draggedCLL);
                }
                else if (draggedCLL instanceof ConfigurationLinearLayout) {
                    Object[] arrows = draggedCLL.getConnectionArrows().toArray();
                   for (Object o : arrows) {
                       ImageViewToImageViewArrow arrow = (ImageViewToImageViewArrow)o;
                       List rules;
                        if (arrow.getSourceLinearLayout() instanceof ConnectorLinearLayout) {
                            ((ConnectorLinearLayout)arrow.getSourceLinearLayout()).removeConfiguration(((ConfigurationLinearLayout) draggedCLL).getConfiguration());
                            rules = ((ConnectorLinearLayout) arrow.getSourceLinearLayout()).getAllRules();
                            if (rules.size() == 0) {
                                configureRules.removeConnectorLinearLayout((ConnectorLinearLayout) arrow.getSourceLinearLayout());
                            }
                        } else if (arrow.getTargetLinearLayout() instanceof ConnectorLinearLayout) {
                            ((ConnectorLinearLayout)arrow.getTargetLinearLayout()).removeConfiguration(((ConfigurationLinearLayout) draggedCLL).getConfiguration());
                            rules =((ConnectorLinearLayout) arrow.getTargetLinearLayout()).getAllRules();
                            if (rules.size() == 0) {
                                configureRules.removeConnectorLinearLayout((ConnectorLinearLayout) arrow.getTargetLinearLayout());
                            }
                        }
                       ((ViewGroup)configureRules.findViewById(R.id.rule_container)).removeView(arrow);
                   }
                    ((ViewGroup)configureRules.findViewById(R.id.rule_container)).removeView(draggedCLL);
                    draggedCLL.removeAllConnectionArrows();
                }
                break;
        }
        return false;
    }
 
    public boolean handleDragWithinRuleContainer(View v, DragEvent event) {
        if (event.getLocalState() instanceof ConnectedLinearLayout) {
            ConnectedLinearLayout draggedCLL = (ConnectedLinearLayout)event.getLocalState();
            switch (event.getAction()) {
                case DragEvent.ACTION_DRAG_LOCATION:
                    draggedCLL.setX(event.getX() - draggedCLL.getWidth() / 2);
                    draggedCLL.setY(event.getY() - draggedCLL.getHeight() / 2);
                    for (ImageViewToImageViewArrow arrow : draggedCLL.getConnectionArrows()) {
                        arrow.update();
                    }
                    break;

                case DragEvent.ACTION_DROP:
                    configureRules.enableRuleContainerScroll();
                    draggedCLL.setX(event.getX() - draggedCLL.getWidth() / 2);
                    draggedCLL.setY(event.getY() - draggedCLL.getHeight() / 2);
                    draggedCLL.setVisibility(View.VISIBLE);
                    for (ImageViewToImageViewArrow arrow : draggedCLL.getConnectionArrows()) {
                        arrow.update();
                    }
                    break;
            }
            return true;
        }
        else {
            return false;
        }
    }

    public boolean handleDragFromIOContainerToRuleContainer(View v, DragEvent event) {
        LinearLayout draggedContainer = (LinearLayout) event.getLocalState();
        int sourceViewId = ((View)draggedContainer.getParent()).getId();
        int objectId = draggedContainer.getId();
        Object object = Search.getObjectById(objectId);
        Device device = null;
        if(object instanceof Device) {
            device = (Device)object;
        }

        switch (event.getAction()) {
            case DragEvent.ACTION_DROP:
                configureRules.enableRuleContainerScroll();
                if(sourceViewId == R.id.input_container && v.getId() == R.id.rule_container) {
                    if(device != null) {
                        Condition newCondition = new Condition(Model.getInstance().getActiveBoard());
                        newCondition.setDevice(device);
                        Point location = new Point((int)event.getX() - draggedContainer.getWidth() / 2,
                                (int)event.getY() - draggedContainer.getHeight() / 2);
                        addConfigurationToRuleContainer(newCondition, location, true);
                    }

                } else if (sourceViewId == R.id.output_container && v.getId() == R.id.rule_container) {
                    if(device != null) {
                        Action newAction = new Action(Model.getInstance().getActiveBoard());
                        newAction.setDevice(device);
                        Point location = new Point((int)event.getX() - draggedContainer.getWidth() / 2,
                                (int)event.getY() - draggedContainer.getHeight() / 2);
                        addConfigurationToRuleContainer(newAction, location, true);
                    }
                }
                break;
            default:
                break;
        }
        return true;
    }

    public ConfigurationLinearLayout addConfigurationToRuleContainer(Configuration configuration, Point location, boolean showAlertDialog) {
        ConfigurationLinearLayout cll = new ConfigurationLinearLayout(configureRules, configuration);
        cll.setOrientation(LinearLayout.VERTICAL);
        cll.setId(configuration.getDevice().getId());
        final float scale = configureRules.getResources().getDisplayMetrics().density;
        int pixels = (int) (ConfigureRules.configurationImageViewSize * scale + 0.5f);
        int paddingPixels = (int) (ConfigureRules.configurationImageViewPaddingSize * scale + 0.5f);
        LinearLayout.LayoutParams params = new LinearLayout.LayoutParams(pixels, ViewGroup.LayoutParams.WRAP_CONTENT);
        cll.setLayoutParams(params);
        cll.setX(location.x);
        cll.setY(location.y);
        cll.setGravity(Gravity.CENTER);

        ImageView imageView = new ImageView(configureRules);
        imageView.setImageResource(ElementConnector.getPictureId(configureRules, configuration.getDevice()));
        params = new LinearLayout.LayoutParams( pixels, pixels);
        imageView.setLayoutParams(params);
        imageView.setPadding(paddingPixels, paddingPixels, paddingPixels, paddingPixels);
        if(configuration instanceof Condition){
            imageView.setBackgroundResource(R.drawable.rectangleboarder);
        } else if(configuration instanceof Action){
            imageView.setBackgroundResource(R.drawable.circleboarder);
        }
        cll.setImage(imageView);

        String configurationText = "";
        if (configuration.getDevice().getText() != null && !configuration.getDevice().getText().equals("")) {
            configurationText = configuration.getDevice().getText() + "\n";
        }
        configurationText += configuration.getBoard().getName();
        TextView textView = new TextView(configureRules);
        textView.setText(configurationText);
        textView.setTextSize(TypedValue.COMPLEX_UNIT_SP, ConfigureRules.textSize);
        textView.setLayoutParams(new LinearLayout.LayoutParams(pixels, ViewGroup.LayoutParams.WRAP_CONTENT));
        textView.setGravity(Gravity.CENTER);
        textView.setTextColor(Color.BLACK);
        textView.setTypeface(null, Typeface.BOLD);
        cll.addView(textView);
        cll.setTextView(textView);

        cll.setOnTouchListener(new ConnectedLinearLayoutOnTouchListener(configureRules));
        configureRules.drawCLLInRuleContainer(cll);
        if(showAlertDialog) {
            configureRules.configureParameters(cll);
        }

        return cll;
    }
}