/*
 * Licensed to UbiCollab.org under one or more contributor
 * license agreements.  See the NOTICE file distributed
 * with this work for additional information regarding
 * copyright ownership. UbiCollab.org licenses this file
 * to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package no.ntnu.android.toolkit.graphic.listener;

import android.support.v4.widget.DrawerLayout;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ListView;
import no.ntnu.android.toolkit.activities.ConfigureRules;
import no.ntnu.android.toolkit.activities.R;
import no.ntnu.android.toolkit.graphic.NavigationDrawer;
import no.ntnu.android.toolkit.model.Board;
import no.ntnu.android.toolkit.model.Model;

public class NavigationDrawerListener implements ListView.OnItemClickListener {
    private ConfigureRules configureRules;
    private NavigationDrawer navigationDrawer;

    public NavigationDrawerListener(ConfigureRules configureRules, NavigationDrawer navigationDrawer) {
        this.configureRules = configureRules;
        this.navigationDrawer = navigationDrawer;
    }

    @Override
    public void onItemClick(AdapterView<?> adapterView, View view, int i, long l) {
        DrawerLayout drawerLayout = (DrawerLayout) configureRules.findViewById(R.id.drawer_layout);
        ListView drawerList = (ListView) configureRules.findViewById(R.id.left_drawer);
        Object object = drawerList.getItemAtPosition(i);
        if(object instanceof Board) {
            Board board = (Board)object;
            if(Model.getInstance().setActive(board)) {
                drawerLayout.closeDrawer(drawerList);
                configureRules.initiateBoardItems();
                navigationDrawer.updateAdapter();
                configureRules.greyOutOtherBoards(board);
            }
        } else {
            throw new IllegalStateException("Selected item in menu list is not a board object");
        }
    }


}