package no.ntnu.android.toolkit.graphic.view;

import android.content.Context;
import android.graphics.Point;
import android.view.View;
import android.widget.ImageView;
import android.widget.LinearLayout;

import java.util.ArrayList;

public class ConnectedLinearLayout extends LinearLayout {

    protected ArrayList<ImageViewToImageViewArrow> connectionArrows;

    private ImageView image;

    private ImageViewToPointArrow touchArrow;

    public ImageViewToPointArrow getTouchArrow() {
        return touchArrow;
    }

    public void setTouchArrow(ImageViewToPointArrow touchArrow) {
        this.touchArrow = touchArrow;
    }

    public ConnectedLinearLayout(Context context) {
        super(context);
        this.connectionArrows = new ArrayList<ImageViewToImageViewArrow>();
    }

    public void addConnectionArrow(ImageViewToImageViewArrow arrow) {
        if (!connectionArrows.contains(arrow)) {
            connectionArrows.add(arrow);
        }
    }

    public void removeAllConnectionArrows() {
        Object[] connectionArrowsArray = connectionArrows.toArray();
        for (Object connectionArrow : connectionArrowsArray) {
            ((ImageViewToImageViewArrow)connectionArrow).disconnectFromConnectedImageViews();
        }
    }

    public ArrayList<ImageViewToImageViewArrow> getConnectionArrows() {
        return connectionArrows;
    }

    public boolean isNeighborOf(ConnectedLinearLayout lookedForCLL) {
        for (ImageViewToImageViewArrow arrow : connectionArrows) {
            if (arrow.getTargetLinearLayout() == lookedForCLL || arrow.getSourceLinearLayout() == lookedForCLL) {
                return true;
            }
        }
        return false;
    }

    public ImageView getImage() {
        return image;
    }

    public void setImage(ImageView image) {
        addView(image);
        this.image = image;
    }

    public Point getArrowSourcePoint() {
        Point source = getCenterOfView(getImage());
        source.x += getX();
        source.y += getY();
        return source;
    }

    public Point getDrawPoint() {
        Point source = new Point();
        source.x = (int)getX();
        source.y = (int)getY();
        return source;
    }

    private Point getCenterOfView(View view) {
        int sourceWidth, sourceHeight;
        //ConnectorLL does not have width and height yet because it is not drawned
        if (!isViewDrawn(view)) {
            sourceWidth = 50;
            sourceHeight = 50;
        } else {
            sourceWidth = view.getWidth();
            sourceHeight = view.getHeight();
        }
        return new Point((int)(view.getX() + sourceWidth / 2), (int)(view.getY() + sourceHeight / 2));
    }

    private boolean isViewDrawn(View view) {
        return view.getWidth() > 0;
    }
}
