package no.ntnu.android.toolkit.graphic.view;


import android.content.Context;
import no.ntnu.android.toolkit.graphic.ConnectorIdGenerator;
import no.ntnu.android.toolkit.model.*;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

public class ConnectorLinearLayout extends ConnectedLinearLayout {


    private HashMap<Board, Rule> boardsSendRule;
    private HashMap<Board, Rule> boardsOtherRule;
    private final int id;

    public ConnectorLinearLayout(Context context) {
        super(context);
        boardsOtherRule = new HashMap<Board, Rule>();
        boardsSendRule = new HashMap<Board, Rule>();
        id = ConnectorIdGenerator.getInstance().getNext();
    }

    public ConnectorLinearLayout(Context context, int id) {
        super(context);
        boardsOtherRule = new HashMap<Board, Rule>();
        boardsSendRule = new HashMap<Board, Rule>();
        this.id = id;
        ConnectorIdGenerator.getInstance().registerIdUsage(id);
    }

    public Rule getBoardsOtherRuleByBoard(Board board){
        return boardsOtherRule.get(board);
    }

    public Rule getBoardsSendRuleByBoard(Board board) {
        return boardsSendRule.get(board);
    }


    public void addRuleToBoardsSendRule(Rule rule) {
        boardsSendRule.put(rule.getBoard(), rule);
        rule.getBoard().addRule(rule);
    }

    public void addRuleToBoardsOtherRule(Rule rule) {
        boardsOtherRule.put(rule.getBoard(), rule);
        rule.getBoard().addRule(rule);
    }

    public void removeConfiguration(Configuration configuration){

        if(configuration instanceof Condition) {
            if (boardsSendRule.get(configuration.getBoard()) != null) {
                boardsSendRule.get(configuration.getBoard()).removeConfiguration(configuration);
                Rule boardSendRule = boardsSendRule.get(configuration.getBoard());
                Object[] actionArray = boardSendRule.getActions().toArray();
                for (Object action : actionArray) {
                    if (action instanceof AutomaticSendMessage) {
                        if (boardsSendRule.get(configuration.getBoard()).getConditions().size() == 0) {
                            boardsSendRule.get(configuration.getBoard()).removeConfiguration((Configuration) action);
                        }
                        removeReceiveMessageByValue(((AutomaticSendMessage) action).getMessage());
                    }
                }

            }
            if (boardsOtherRule.get(configuration.getBoard()) != null) {
                boardsOtherRule.get(configuration.getBoard()).removeConfiguration(configuration);
            }
        }
        else if(configuration instanceof Action) {
            Rule boardOtherRule = boardsOtherRule.get(configuration.getBoard());
            boardOtherRule.removeConfiguration(configuration);
            Object[] conditionArray = boardOtherRule.getConditions().toArray();
            for(Object condition : conditionArray) {
                if (condition instanceof AutomaticReceiveMessage) {
                    if (boardOtherRule.getActions().size() == 0) {
                        boardOtherRule.removeConfiguration((Configuration) condition);
                    }
                    removeSendMessageByValue(((AutomaticReceiveMessage) condition).getMessage());
                }
            }
        }

        Object [] otherRulesArray = getAllOtherRules().toArray();
        for(Object rule : otherRulesArray){
            if(((Rule)rule).getActions().size() == 0 || ((Rule)rule).getConditions().size() == 0){
                boardsOtherRule.remove(((Rule) rule).getBoard());
                ((Rule)rule).getBoard().removeRule((Rule) rule);
            }
        }

        Object [] sendRulesArray = getAllSendRules().toArray();
        for(Object rule : sendRulesArray){
            if(((Rule)rule).getActions().size() == 0 || ((Rule)rule).getConditions().size() == 0){
                boardsSendRule.remove(((Rule) rule).getBoard());
                ((Rule)rule).getBoard().removeRule((Rule)rule);
            }
        }

    }

    private void removeReceiveMessageByValue(String receiveMessageValue){
        for(Rule rule : getAllOtherRules()) {
            Object [] conditionsArray = rule.getConditions().toArray();
            for (Object condition : conditionsArray) {
                if(condition instanceof AutomaticReceiveMessage){
                    if(((AutomaticReceiveMessage) condition).getMessage().equalsIgnoreCase(receiveMessageValue)){
                        if(!receiveMessageHasCorrespondingSendMessage((AutomaticReceiveMessage) condition)){
                            rule.removeConfiguration((Condition)condition);
                        }
                    }
                }
            }
        }

    }

    private void removeSendMessageByValue(String sendMessageValue){
        for(Rule rule : getAllSendRules()) {
            Object [] actionsArray = rule.getActions().toArray();
            for (Object sendMessage : actionsArray) {
                if(((AutomaticSendMessage)sendMessage).getMessage().equalsIgnoreCase(sendMessageValue)){
                    if(!sendMessageHasCorrespondingReceiveMessage((AutomaticSendMessage) sendMessage)){
                        rule.removeConfiguration((Action)sendMessage);
                    }
                }
            }
        }
    }

    private boolean receiveMessageHasCorrespondingSendMessage(AutomaticReceiveMessage receiveMessage){

        for(Rule rule : getAllSendRules()){
            for(Action action : rule.getActions()) {
                if (action instanceof AutomaticSendMessage) {
                    if (((AutomaticSendMessage) action).getMessage().equalsIgnoreCase(receiveMessage.getMessage())) {
                        return true;
                    }
                }
            }
        }
        return false;
    }

    private boolean sendMessageHasCorrespondingReceiveMessage(AutomaticSendMessage sendMessage) {
        for (Rule rule : getAllOtherRules()) {
            for (Condition condition : rule.getConditions()) {
                if (condition instanceof AutomaticReceiveMessage) {
                    if(((AutomaticReceiveMessage) condition).getMessage().equalsIgnoreCase(sendMessage.getMessage())){
                        return true;
                    }
                }
            }
        }
        return false;
    }

    public List<ConfigurationLinearLayout> getAllActionLinearLayouts(){
        List<ConfigurationLinearLayout> actionLLs = new ArrayList<ConfigurationLinearLayout>();
        for(ImageViewToImageViewArrow imageViewToImageViewArrow : getConnectionArrows()){
            if(imageViewToImageViewArrow.getTargetLinearLayout() instanceof ConfigurationLinearLayout){
                actionLLs.add((ConfigurationLinearLayout) imageViewToImageViewArrow.getTargetLinearLayout());
            }
        }
        return actionLLs;
    }

    public List<ConfigurationLinearLayout> getAllConditionLinearLayouts(){
        List<ConfigurationLinearLayout> conditionLLs = new ArrayList<ConfigurationLinearLayout>();
        for(ImageViewToImageViewArrow imageViewToImageViewArrow : getConnectionArrows()){
            if(imageViewToImageViewArrow.getSourceLinearLayout() instanceof  ConfigurationLinearLayout){
                conditionLLs.add((ConfigurationLinearLayout) imageViewToImageViewArrow.getSourceLinearLayout());
            }
        }
        return conditionLLs;
    }

   public boolean boardsSendRuleContainingSendMessage(Board board, AutomaticSendMessage sendMessage) {
       for (Action action :  boardsSendRule.get(board).getActions()) {
           if (action instanceof AutomaticSendMessage && board == sendMessage.getBoard()) {
               if(((AutomaticSendMessage) action).getMessage()
                       .equalsIgnoreCase(sendMessage.getMessage())){
                   return true;
               }
           }
       }
       return false;
   }

    public boolean boardsOtherRuleContainingReceiveMessage(Board board, AutomaticReceiveMessage receiveMessage) {
        for (Condition condition :  boardsOtherRule.get(board).getConditions()) {
            if (condition instanceof AutomaticReceiveMessage &&  board == receiveMessage.getBoard()) {
                if(((AutomaticReceiveMessage) condition).getMessage()
                        .equalsIgnoreCase(receiveMessage.getMessage())){
                    return true;
                }
            }
        }
        return false;
    }

   public List<Rule> getAllRules(){
       List<Rule> rules = new ArrayList<Rule>();

       for(Board board : boardsOtherRule.keySet()){
           rules.add(boardsOtherRule.get(board));
       }
       for(Board board : boardsSendRule.keySet()){
           rules.add(boardsSendRule.get(board));
       }

       return rules;
   }

    public int getId(){
        return id;
    }

    private List<Rule> getAllOtherRules() {
        List<Rule> rules = new ArrayList<Rule>();
        for (Board board : boardsOtherRule.keySet()) {
            rules.add(boardsOtherRule.get(board));
        }
        return rules;
    }

    private List<Rule> getAllSendRules(){
        List<Rule> rules = new ArrayList<Rule>();
        for(Board board : boardsSendRule.keySet()){
            rules.add(boardsSendRule.get(board));
        }
        return rules;
    }

    public List<Board> getBoards() {
        List<Board> boards = new ArrayList<Board>();
        for (Board board : boardsSendRule.keySet()) {
            if (!boards.contains(board)) {
                boards.add(board);
            }
        }
        for (Board board : boardsOtherRule.keySet()) {
            if (!boards.contains(board)) {
                boards.add(board);
            }
        }
        return boards;
    }

    public boolean canConnect(ConfigurationLinearLayout cll) {
        List<Board> boards = getBoards();
        if (cll.getConfiguration().getBoard().getWifiDevice() == null) {
            if (boards.size() == 1 && cll.getConfiguration().getBoard() != boards.get(0)) {
                return false;
            }
            if (boards.size() > 1) {
                return false;
            }
        } else {
            if (boards.size() == 1 && boards.get(0).getWifiDevice() == null) {
                return false;
            }
        }
        return true;
    }
}
