/*
 * Licensed to UbiCollab.org under one or more contributor
 * license agreements.  See the NOTICE file distributed
 * with this work for additional information regarding
 * copyright ownership. UbiCollab.org licenses this file
 * to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package no.ntnu.android.toolkit.model;

import java.io.Serializable;

public abstract class Configuration implements IdInterface, Serializable {

    private Device device;
    private DeviceFunction deviceFunction;
    private final int id;
    private Board board;

    public int getId() {
        return id;
    }


    public Configuration() {
        id = IdGenerator.getInstance().getNext(this);
    }

    public Configuration(DeviceFunction deviceFunction, Board board) {
        this();
        this.board = board;
        this.deviceFunction = deviceFunction;
    }

    public void setDevice(Device device) {
        this.device = device;
    }

    public Device getDevice() {
        return device;
    }

    public void setBoard(Board board) {
        this.board = board;
    }

    public Board getBoard() {
        return board;
    }

    public void setDeviceFunction(DeviceFunction deviceFunction) {
        this.deviceFunction = deviceFunction;
    }

    public DeviceFunction getDeviceFunction() {
        return deviceFunction;
    }
}