/*
 * Licensed to UbiCollab.org under one or more contributor
 * license agreements.  See the NOTICE file distributed
 * with this work for additional information regarding
 * copyright ownership. UbiCollab.org licenses this file
 * to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package no.ntnu.android.toolkit.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

public class Device implements IdInterface, Serializable {

    private String type;
    private String text;
    private List<Pin> pins;
    private List<DeviceFunction> deviceFunctions;
    private final int id;

    public Device() {
        deviceFunctions = new ArrayList<DeviceFunction>();
        pins = new ArrayList<Pin>();
        text = "";
        id = IdGenerator.getInstance().getNext(this);
    }

    public Device(String type, List<Pin> pins) {
        this();
        this.type = type;
        if(pins != null) {
            this.pins = pins;
        }
    }

    public void setType(String type) {
        this.type = type;
    }

    public String getType() {

        return type;
    }

    public String getText() {
        return text;
    }

    public void setText(String text) {
        this.text = text;
    }

    public void addPin(Pin pin) {
        pins.add(pin);
    }

    public DeviceFunction createNewDeviceFunctionByName(String name) {
        for(DeviceFunction deviceFunction : deviceFunctions) {
            if (deviceFunction.getGuiName().equalsIgnoreCase(name)) {
                List<Parameter> parametersCopy = new ArrayList<Parameter>();
                Parameter parameterCopy;
                for(Parameter parameter : deviceFunction.getParameters()) {
                    parameterCopy = new Parameter(parameter.getUiElementName(), parameter.getDatatypeName(), parameter.getLabel());
                    parametersCopy.add(parameterCopy);
                }
                return new DeviceFunction(deviceFunction.getName(), deviceFunction.getGuiName() ,parametersCopy,
                        deviceFunction.isContinuous(), deviceFunction.getType());
            }
        }
        return null;
    }

    public List<Pin> getPins() {
        return pins;
    }

    public String toString() {
        return type + "(" + pins + ")";
    }

    public List<DeviceFunction> getDeviceFunctions() {
        return deviceFunctions;
    }

    public void setDeviceFunctions(List<DeviceFunction> deviceFunctions){
        this.deviceFunctions = deviceFunctions;
    }

    public void addDeviceFunction(DeviceFunction deviceFunction){
        deviceFunctions.add(deviceFunction);
    }

    @Override
    public int getId() {
        return id;
    }
}