/*
 * Licensed to UbiCollab.org under one or more contributor
 * license agreements.  See the NOTICE file distributed
 * with this work for additional information regarding
 * copyright ownership. UbiCollab.org licenses this file
 * to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package no.ntnu.android.toolkit.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

public class DeviceFunction implements Serializable {

    private String name;
    private String guiName;
    private List<Parameter> parameters;
    private boolean continuous;
    private Type type;

    public boolean isContinuous() {
        return continuous;
    }

    public void setContinuous(boolean continuous) {
        this.continuous = continuous;
    }

    public DeviceFunction(Type type) {
        parameters = new ArrayList<Parameter>();
        this.type = type;
    }

    public DeviceFunction(String name, Type type) {
        this(type);
        this.name = name;
    }

    public DeviceFunction(String name, String guiName, List<Parameter> parameters, boolean continuous, Type type) {
        this(name, type);
        this.continuous = continuous;
        this.guiName = guiName;
        if (parameters != null) {
            this.parameters = parameters;
        }
    }

    public void addParameter(Parameter parameter) {
        parameters.add(parameter);
    }

    public List<Parameter> getParameters() {
        return parameters;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getName() {
        return name;
    }

    public String getGuiName() {
        return guiName;
    }

    public void setGuiName(String guiName) {
        this.guiName = guiName;
    }

    public Type getType() {
        return type;
    }

    public void setType(Type type) {
        this.type = type;
    }

    public enum Type {
        SENSOR, ACTUATOR;
    }
}