/*
 * Licensed to UbiCollab.org under one or more contributor
 * license agreements.  See the NOTICE file distributed
 * with this work for additional information regarding
 * copyright ownership. UbiCollab.org licenses this file
 * to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package no.ntnu.android.toolkit.model;


import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

public class HardwareSetup implements IdInterface, Serializable, Iterable<Device> {

    private String name;
    private String description;
    private List<Device> devices;
    private List<RFIDGroup> rfidGroups;
    private final int id;

    public HardwareSetup() {
        devices = new ArrayList<Device>();
        rfidGroups = new ArrayList<RFIDGroup>();
        id = IdGenerator.getInstance().getNext(this);
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getName() {
        return name;
    }
    public List<Device> getDevices() { return devices; }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public void add(Device device) {
        if (!devices.contains(device)) {
            devices.add(device);
        }
    }
    public String toString() {
        String builder = name + ": \n";
        if (devices.size() > 0) {
            for (Device device : devices) {
                builder += device + "\n";
            }
        }
        return builder;
    }

    @Override
    public Iterator<Device> iterator() {
        return devices.iterator();
    }
    public void addRFIDGroup(RFIDGroup rfidGroup){
        rfidGroups.add(rfidGroup);
    }
    public List<RFIDGroup> getRFIDGroups(){
        return rfidGroups;
    }

    public RFIDGroup getRFIDGroupByName(String name){
        for(RFIDGroup rfidGroup : rfidGroups){
            if(rfidGroup.getName().equalsIgnoreCase(name)){
                return rfidGroup;
            }
        }
        return null;
    }

    @Override
    public int getId() {
        return id;
    }

}