/*
 * Licensed to UbiCollab.org under one or more contributor
 * license agreements.  See the NOTICE file distributed
 * with this work for additional information regarding
 * copyright ownership. UbiCollab.org licenses this file
 * to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package no.ntnu.android.toolkit.model;

import no.ntnu.android.toolkit.tool.Search;
import no.ntnu.android.toolkit.reader.BoardTypeReader;
import no.ntnu.android.toolkit.reader.DeviceReader;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;

public class IdGenerator {

    private List<IdInterface> allObjectsWithId;
    private AtomicInteger atomicInteger;
    private static IdGenerator idGenerator;

    private IdGenerator(){
        allObjectsWithId = new ArrayList<IdInterface>();
        atomicInteger = new AtomicInteger(1);
    }

    public List<IdInterface> getAllObjects() {
        return new ArrayList<IdInterface>(allObjectsWithId);
    }

    public static IdGenerator getInstance() {
        if (idGenerator == null) {
            idGenerator = new IdGenerator();
        }
        return idGenerator;
    }

    public int getNext(IdInterface user) {
        if (user.getId() > 0) {
            IdInterface searchResult = Search.getObjectById(user.getId());
            if (searchResult == null) {
                allObjectsWithId.add(user);
                if (atomicInteger.get() < user.getId()) {
                    atomicInteger.set(user.getId() + 1);
                }
                return atomicInteger.getAndIncrement();
            } else if (searchResult == user) {
                //It's ok
                return atomicInteger.getAndIncrement();
            } else {
                throw new IllegalStateException("Two object can not hold the same id: " + user.getId());
            }
        }
        allObjectsWithId.add(user);
        return atomicInteger.incrementAndGet();
    }

    public void flush() {
        allObjectsWithId = new ArrayList<IdInterface>();
        atomicInteger = new AtomicInteger(1);
        DeviceReader.flushed = true;
        BoardTypeReader.flushed = true;
    }
}