/*
 * Licensed to UbiCollab.org under one or more contributor
 * license agreements.  See the NOTICE file distributed
 * with this work for additional information regarding
 * copyright ownership. UbiCollab.org licenses this file
 * to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package no.ntnu.android.toolkit.model;
import java.util.ArrayList;
import java.io.Serializable;

public class Rule implements IdInterface, Serializable {

    private ArrayList<Condition> conditions;
    private ArrayList<String> operators;
    private ArrayList<Action> actions;
    private Board board;
    private final int id;

    public Rule(Board board){
        conditions = new ArrayList<Condition>();
        actions = new ArrayList<Action>();
        operators = new ArrayList<String>();
        this.board = board;
        id = IdGenerator.getInstance().getNext(this);
    }

    public ArrayList<Condition> getConditions() {
        return conditions;
    }

    public ArrayList<String> getOperators() {
        return operators;
    }

    public ArrayList<Action> getActions() {
        return actions;
    }

    public ArrayList<Configuration> getConfigurations() {
        ArrayList<Configuration> list = new ArrayList<Configuration>();
        list.addAll(conditions);
        list.addAll(actions);
        return list;
    }

    public void addConfiguration(Configuration configuration){
        if(configuration instanceof Action) {
            if(!getActions().contains(configuration)){
                actions.add((Action) configuration);
            }
        } else if(configuration instanceof Condition) {
            if(!getConditions().contains(configuration)){
                conditions.add((Condition) configuration);
                if(conditions.size() > 1){
                    addOperator("&&");
                }
            }
        }
    }

    public void removeConfiguration(Configuration configuration) {
        if (configuration instanceof Condition) {
            if (conditions.indexOf(configuration) == 0 && conditions.size() > 1) {
                operators.remove(0);
            } else if(conditions.indexOf(configuration) != -1 && conditions.size() > 1){
                operators.remove(conditions.indexOf(configuration) - 1);
            }
            conditions.remove(configuration);
        } else if (configuration instanceof Action) {
            actions.remove(configuration);
        }
    }

    public void addOperator(String operator) {
        operators.add(operator);
    }

    @Override
    public String toString() {
        StringBuilder builder = new StringBuilder();
        builder.append("ID: " + (this.getId()));
        builder.append(" Conditions: ");
        for(Condition condition : conditions){
            builder.append(condition.toString() + "  ");
        }
        builder.append("\tActions: ");
        for(Action action : actions){
            builder.append(action.toString() + "  ");
        }
        return  builder.toString();
    }

    public Board getBoard() {
        return board;
    }

    public void setBoard(Board board) {
        this.board = board;
    }

    @Override
    public int getId() {
        return id;
    }
}