/*
 * Licensed to UbiCollab.org under one or more contributor
 * license agreements.  See the NOTICE file distributed
 * with this work for additional information regarding
 * copyright ownership. UbiCollab.org licenses this file
 * to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package no.ntnu.android.toolkit.model;

import java.util.ArrayList;
import java.util.List;

public class WifiSettings {

    private String ipAddress;
    private String ssid;
    private String password;
    private List<IpAddressChangeListener> listenerList = new ArrayList<IpAddressChangeListener>();

    public WifiSettings() {
    }

    public WifiSettings(String ssid, String password) {
        this.ssid = ssid;
        this.password = password;
    }

    public String getIpAddress() {
        return ipAddress;
    }

    public void setIpAddress(String ipAddress) {
        this.ipAddress = ipAddress;
        for (IpAddressChangeListener listener : listenerList) {
            listener.ipChanged();
        }
    }

    public String getSsid() {
        return ssid;
    }

    public void setSsid(String ssid) {
        this.ssid = ssid;
    }

    public String getPassword() {
        return password;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public void addIpAddressChangeListener(IpAddressChangeListener ipAddressChangeListener) {
        listenerList.add(ipAddressChangeListener);
    }

    public void removeIpAddressChangeListener(IpAddressChangeListener ipAddressChangeListener) {
        listenerList.remove(ipAddressChangeListener);
    }
}