/*
 * Licensed to UbiCollab.org under one or more contributor
 * license agreements.  See the NOTICE file distributed
 * with this work for additional information regarding
 * copyright ownership. UbiCollab.org licenses this file
 * to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package no.ntnu.android.toolkit.parser;

import no.ntnu.android.toolkit.model.*;
import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;
import org.xmlpull.v1.XmlPullParserFactory;

import java.io.IOException;
import java.io.InputStream;

public class DeviceParser implements Parser {

    private String text;

    public HardwareSetup parse(InputStream inputStream) {
        XmlPullParserFactory factory;
        XmlPullParser parser;
        String tagname;

        HardwareSetup hardwareSetup = new HardwareSetup();

        Device ongoingDevice = null;
        DeviceFunction ongoingDeviceFunction = null;
        Parameter ongoingParameter = null;

        try{
            factory = XmlPullParserFactory.newInstance();
            factory.setNamespaceAware(true);
            parser = factory.newPullParser();
            parser.setInput(inputStream, null);

            int eventType = parser.getEventType();
            while (eventType != XmlPullParser.END_DOCUMENT) {
                tagname = parser.getName();
                switch (eventType) {
                    case XmlPullParser.START_TAG:
                        if(tagname.equalsIgnoreCase("device")) {
                            ongoingDevice = new Device();
                            hardwareSetup.add(ongoingDevice);
                        } else if(tagname.equalsIgnoreCase("parameter")) {
                            ongoingParameter = new Parameter();
                            ongoingDeviceFunction.addParameter(ongoingParameter);
                        } else if(tagname.equalsIgnoreCase("condition")) {
                            ongoingDeviceFunction = new DeviceFunction(DeviceFunction.Type.SENSOR);
                        } else if(tagname.equalsIgnoreCase("action")) {
                            ongoingDeviceFunction = new DeviceFunction(DeviceFunction.Type.ACTUATOR);
                        }
                        break;

                    case XmlPullParser.TEXT:
                        text = parser.getText();
                        break;

                    case XmlPullParser.END_TAG:
                        if (tagname.equalsIgnoreCase("type")) {
                            ongoingDevice.setType(text);

                        } else if (tagname.equalsIgnoreCase("continuous")) {
                            if (text.equalsIgnoreCase("true")) {
                                ongoingDeviceFunction.setContinuous(true);
                            }

                        } else if (tagname.equalsIgnoreCase("name")) {
                            ongoingDeviceFunction.setName(text);

                        } else if(tagname.equalsIgnoreCase("guiName")){
                            ongoingDeviceFunction.setGuiName(text);

                        } else if(tagname.equalsIgnoreCase("label")){
                            ongoingParameter.setLabel(text);
                        }else if (tagname.equalsIgnoreCase("datatype")) {
                            ongoingParameter.setDatatypeName(text);

                        } else if (tagname.equalsIgnoreCase("uiElement")) {
                            ongoingParameter.setUiElementName(text);

                        } else if (tagname.equalsIgnoreCase("action") ||
                                tagname.equalsIgnoreCase("condition")) {
                            ongoingDevice.addDeviceFunction(ongoingDeviceFunction);
                        }
                        break;

                    default:
                        break;
                }
                eventType = parser.next();
            }
        } catch (XmlPullParserException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }

        return hardwareSetup;
    }
}
