/*
 * Licensed to UbiCollab.org under one or more contributor
 * license agreements.  See the NOTICE file distributed
 * with this work for additional information regarding
 * copyright ownership. UbiCollab.org licenses this file
 * to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package no.ntnu.android.toolkit.parser;

import no.ntnu.android.toolkit.model.Device;
import no.ntnu.android.toolkit.model.HardwareSetup;
import no.ntnu.android.toolkit.model.Pin;
import no.ntnu.android.toolkit.model.RFIDGroup;
import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;
import org.xmlpull.v1.XmlPullParserFactory;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

public class HardwareSetupParser implements Parser {

    private Device tempDevice;
    private String text;
    private RFIDGroup rfidGroup;

    public Object parse(InputStream inputstream) {
        XmlPullParserFactory factory;
        XmlPullParser parser;
        HardwareSetup hardwareSetup = new HardwareSetup();
        Pin pin = null;

        try {
            factory = XmlPullParserFactory.newInstance();
            factory.setNamespaceAware(true);
            parser = factory.newPullParser();
            parser.setInput(inputstream, null);

            int eventType = parser.getEventType();
            while (eventType != XmlPullParser.END_DOCUMENT) {
                String tagname = parser.getName();
                switch (eventType) {
                    case XmlPullParser.START_TAG:
                        if (tagname.equalsIgnoreCase("device")) {
                            tempDevice = new Device();
                        } else if (tagname.equalsIgnoreCase("pins")) {
                            pin = new Pin();
                            tempDevice.addPin(pin);
                        } else if (tagname.equalsIgnoreCase("Group")) {
                            rfidGroup = new RFIDGroup();
                        }
                        break;

                    case XmlPullParser.TEXT:
                        text = parser.getText();
                        break;

                    case XmlPullParser.END_TAG:
                        if (tagname.equalsIgnoreCase("name")) {
                            hardwareSetup.setName(text);
                        } else if(tagname.equalsIgnoreCase("description")){
                            hardwareSetup.setDescription(text);
                        } else if (tagname.equalsIgnoreCase("device")) {
                            hardwareSetup.add(tempDevice);
                        } else if (tagname.equalsIgnoreCase("type")) {
                            tempDevice.setType(text);
                        } else if (tagname.equalsIgnoreCase("pinmode")) {
                            pin.setMode(Pin.PinMode.getMode(text));
                        } else if (tagname.equalsIgnoreCase("pinnumber")) {
                            pin.setNumber(text);
                        } else if (tagname.equalsIgnoreCase("text")) {
                            tempDevice.setText(text);
                        } else if (tagname.equalsIgnoreCase("RFIDName")) {
                            rfidGroup.setName(text);
                        } else if (tagname.equalsIgnoreCase("Tag")) {
                            rfidGroup.addTag(text);
                        } else if(tagname.equalsIgnoreCase("Group")){
                            hardwareSetup.addRFIDGroup(rfidGroup);
                        }
                        break;

                    default:
                        break;
                }
                eventType = parser.next();
            }
        } catch (XmlPullParserException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return hardwareSetup;
    }
}