/*
 * Licensed to UbiCollab.org under one or more contributor
 * license agreements.  See the NOTICE file distributed
 * with this work for additional information regarding
 * copyright ownership. UbiCollab.org licenses this file
 * to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package no.ntnu.android.toolkit.reader;

import android.app.Activity;
import android.os.AsyncTask;
import no.ntnu.android.toolkit.activities.R;
import no.ntnu.android.toolkit.model.Board;
import no.ntnu.android.toolkit.model.Device;
import no.ntnu.android.toolkit.model.HardwareSetup;
import no.ntnu.android.toolkit.model.Model;
import no.ntnu.android.toolkit.parser.FolderScanner;
import no.ntnu.android.toolkit.parser.HardwareSetupParser;

import java.util.List;

public class BoardTypeReader {

    private FolderScanner folderScanner;
    private Activity activity;
    private List<HardwareSetup> devices = null;
    public static boolean flushed;

    public BoardTypeReader(Activity startFromActivity) {
        this.activity = startFromActivity;
    }

    public List<HardwareSetup> getDevices() {
        if (devices == null || flushed) {
            load();
            flushed = false;
        }
        return devices;
    }

    private void load() {
        ReadIOAsync readIOAsync = new ReadIOAsync();
        readIOAsync.execute();
        try {
            readIOAsync.get();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    public static void addDeviceFunctionsToBoards(Activity activity) {
        DeviceReader deviceReader = new DeviceReader(activity);
        HardwareSetup hardwareSetup = deviceReader.getHardwareSetup();
        List<Board> boards = Model.getInstance().getBoards();
        for(Device hardwareDevice : hardwareSetup.getDevices()) {
            for (Board board : boards) {
                for(Device boardDevice : board.getHardwareSetup().getDevices()) {
                    if(boardDevice.getType().equalsIgnoreCase(hardwareDevice.getType())) {
                        boardDevice.setDeviceFunctions(hardwareDevice.getDeviceFunctions());
                    }
                }
            }
        }
    }

    //Reads the board types in background
    class ReadIOAsync extends AsyncTask<Void, Void, Void> {

        @Override
        protected Void doInBackground(Void... params) {
            folderScanner = new FolderScanner(
                    activity.getResources().getAssets(), new HardwareSetupParser(), activity.getResources().getString(R.string.boardTypeFolder), ".xml");
            devices = folderScanner.getObjectFromFiles();
            return null;
        }

        @Override
        protected void onPreExecute() {
            super.onPreExecute();
        }

        @Override
        protected void onPostExecute(Void result) {
            super.onPostExecute(result);
        }
    }
}
