/*
 * Licensed to UbiCollab.org under one or more contributor
 * license agreements.  See the NOTICE file distributed
 * with this work for additional information regarding
 * copyright ownership. UbiCollab.org licenses this file
 * to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package no.ntnu.android.toolkit.reader;

import android.app.Activity;
import android.os.AsyncTask;
import no.ntnu.android.toolkit.activities.R;
import no.ntnu.android.toolkit.model.HardwareSetup;
import no.ntnu.android.toolkit.parser.DeviceParser;
import no.ntnu.android.toolkit.parser.FolderScanner;

import java.util.List;

public class DeviceReader {

    private FolderScanner folderScanner;
    private HardwareSetup hardwareSetup;
    public static boolean flushed;

    public DeviceReader(Activity startFromActivity) {
        folderScanner = new FolderScanner(
                startFromActivity.getResources().getAssets(), new DeviceParser(),
                startFromActivity.getResources().getString(R.string.deviceFolder), ".xml");
    }

    public HardwareSetup getHardwareSetup() {
        if (hardwareSetup == null || flushed) {
            load();
            flushed = false;
        }
        return hardwareSetup;
    }

    public void load() {
        ReadIOAsync readIOAsync = new ReadIOAsync();
        readIOAsync.execute();
        try {
            readIOAsync.get();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    //Reads in background
    class ReadIOAsync extends AsyncTask<Void, Void, Void> {

        @Override
        protected Void doInBackground(Void... params) {
            List list = folderScanner.getObjectFromFiles();
            //should only be one element in this list
            if(list.size() == 1 &&  list.get(0) instanceof HardwareSetup) {
                hardwareSetup = (HardwareSetup)list.get(0);
            } else {
                throw new IllegalStateException("Read hardwareSetup was " + list.size() + "(should be 1)");
            }
            return null;
        }

        @Override
        protected void onPreExecute() {
            super.onPreExecute();
        }

        @Override
        protected void onPostExecute(Void result) {
            super.onPostExecute(result);
        }
    }
}